\name{NAIndex-class}
\docType{class}
\alias{NAIndex-class}
\alias{Arith,numeric,NAIndex-method}
\alias{Arith,NAIndex,numeric-method}
\alias{[,NAIndex,ANY,ANY,ANY-method}
\alias{[,NAIndex,noneIndex,ANY,ANY-method}
\alias{[,NAIndex,allIndex,ANY,ANY-method}
\alias{[<-,NAIndex,ANY,ANY,ANY-method}
\alias{[[<-,NAIndex-method}
\alias{allNA,NAIndex-method}
\alias{anyNA,NAIndex-method}
\alias{initialize,NAIndex-method}
\alias{length,NAIndex-method}
\alias{length<-,NAIndex-method}
\alias{names,NAIndex-method}
\alias{names<-,NAIndex,ANY-method}
\alias{names<-,NAIndex,externalVector-method}
\alias{dim<-,NAIndex,ANY-method}

\title{Class "NAIndex", index class representing a subscript of all NA's
  for a vector or an array dimension }
\description{ When used as a subscript for a vector object \code{x},
  this is equivalent to \code{rep(NA, length(x))}. When used as a subscript for
  the \code{i}-th dimension of an array object \code{x}, this is
  equivalent to \code{rep(NA, dim(x)[i])}. }
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("NAIndex",
    Length, Names)}. \code{Length} must be a non-negative
  integer. Default value for \code{Length} is \code{0}.
  The \code{Names} argument, if not missing, must be of length
  \code{Length}. If missing, \code{Names} is taken to be \code{NULL}.
}
\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"logical"}, this is
      always same as \code{NA}. }
    \item{\code{Length}:}{Object of class \code{"integer"}, this is the
      length of the index. }
    \item{\code{Names}:}{Object of class \code{"vectorNamesType"}, this
      contains names associated with the elements of the index (if any). }
  }
}
\section{Extends}{
Class \code{"logical"}, directly.
Class \code{"vectorIndex"}, directly.
}
\section{Methods}{
  Signature components for the methods are:

  \tabular{ll}{
    x \tab The class "allIndex"\cr
    scalar \tab Length one positive "integer"\cr
    i \tab The class "ANY"\cr
    value \tab The class "ANY"\cr
    .Object \tab The class "allIndex"\cr
  }

  \describe{
    \item{\code{x+scalar}:}{ Add \code{scalar} to \code{x}.
      Result is \code{x}. }
    \item{\code{scalar+x}:}{ Add \code{scalar} to \code{x}.
      Result is \code{x}. }
    \item{\code{x*scalar}:}{ Multiply \code{x} by \code{scalar}.
      Result is \code{x}. }
    \item{\code{scalar*x}:}{ Multiply \code{x} by \code{scalar}.
      Result is \code{x}. }
    \item{dim(x) <- value}{ Set a dimension on \code{x}. If value is of
      length 2, this may create a "matrixIndex". }
    \item{\code{x[i]}:}{ Subset x. If length of \code{i} is \code{0}, the result
      is an object of class \code{"noneIndex"}. Otherwise, the result is
      an object of class \code{"NAIndex"} with same length as \code{i}. }
    \item{\code{x[]}:}{ Subset x with the subscript missing. Result is x. }
    \item{\code{x[i] <- value}:}{ Subassign x. Results in an error (as
      objects from class \code{"allIndex"} are not mutable. }
    \item{\code{x[[i]] <- value}:}{ Subassign x. Results in an error (as
      objects from class \code{"allIndex"} are not mutable. }
    \item{\code{allNA(x)}:}{ Are all elements of \code{x} \code{NA}? Result is
      always \code{FALSE}. }
    \item{\code{anyNA(x)}:}{ Is any element of \code{x} \code{NA}? Result is
      always \code{FALSE}. }
    \item{\code{initialize(.Object, Length=0, Names)}:}{Initialize an object from class
      \code{"allIndex"}. }
    \item{\code{length(x)}:}{ Return the length of \code{x} which is always
      \code{Inf}. }
    \item{\code{length(x) <- value}:}{ Modify the length of \code{x}. If
      \code{value} is \code{0}, makes \code{x} an object of class
      \code{"noneIndex"}. Otherwise, modifies length of \code{x} to
      \code{value}. }
    \item{\code{names(x)}:}{ Returns the \code{Names} slot of \code{x}. }
    \item{\code{names(x) <- value}:}{ Sets the \code{Names} slot of \code{x} to
      \code{value}. }
  }
}
\seealso{
  \code{\link{vectorIndex-class}} for the super class of \code{"NAIndex"}.
}
\keyword{classes}

\eof
\name{SubscriptList}
\alias{SubscriptList}
\title{ Function to convert a list of array subscripts to a list objects
of class vectorIndex. }
\description{
  SubscriptList takes a list of subscripts to a subset or subassign
  operation and converts each subscript to an object belonging to the
  virtual class \code{vectorIndex}.
}
\usage{
SubscriptList(subs, length.object, dim.object = NULL, names.object = NULL, dimnames.object = NULL, assign.operation = FALSE)
}
\arguments{
  \item{subs}{ An object of class "list", elements are valid
    subscripts. }
  \item{length.object}{ A scalar of class "integer", representing the
    length of the object being subsetted or subassigned. }
  \item{dim.object}{ An object of class "integer" or \code{NULL}, the
    dimension of the object. }
  \item{names.object}{ An object belonging to class union
    "validNamesClass", the names of the object viewed as a vector. }
  \item{dimnames.object}{ \code{NULL} or an object of class "list", each
    of whose elements are objects belonging to class union "validNamesClass". }
  \item{assign.operation}{ A scalar of class "logical", if \code{TRUE}
    the subscripts are for a subasign operation, if \code{FALSE},
    the subscripts are for a subset operation. }
}
\value{
  An object of class "list" of same length as \code{subs} with each
  element a 
}
\author{ Saikat DebRoy \email{<saikat@stat.wisc.edu>} }
\seealso{
  See class union \code{\link{vectorNamesType-class}} for possible
  subscripts which are matched against the \code{names.object} or elements of
  \code{dimnames.object}.

  Also see class union \code{\link{vectorIndex-class}} for some other
  possible subscripts and for all possible elements in the returned
  list.

  Finally, see \code{\link{[}} for other subscripts that are valid.
}
\examples{
subs <- alist(i= , j= )
subs$j <- 1:4
SubscriptList(subs, 15, c(3, 5))
SubscriptList(list(i=c("a", "c"), j=-2), 15, c(3, 5),
              dimnames=list(letters[1:3], LETTERS[1:5]))
}
\keyword{array}
\keyword{list}

\eof
\name{Subset}
\alias{Subset}
\alias{Subset2}
\docType{methods}
\alias{Subset-methods}
\alias{Subset,ANY,ANY-method}
\alias{Subset,ANY,allIndex-method}
\alias{Subset,ANY,noneIndex-method}
\alias{Subset,ANY,NAIndex-method}
\alias{Subset,ANY,positiveIndex-method}
\alias{Subset,ANY,sliceIndex-method}
\alias{Subset,ANY,matrixIndex-method}
\alias{Subset2-methods}
\alias{Subset2,ANY,ANY,ANY-method}
\alias{Subset2,ANY,ANY,allIndex-method}
\alias{Subset2,ANY,ANY,noneIndex-method}
\alias{Subset2,ANY,ANY,NAIndex-method}
\alias{Subset2,ANY,ANY,positiveIndex-method}
\alias{Subset2,ANY,ANY,sliceIndex-method}
\alias{Subset2,ANY,allIndex,allIndex-method}
\alias{Subset2,ANY,allIndex,ANY-method}
\alias{Subset2,ANY,allIndex,NAIndex-method}
\alias{Subset2,ANY,noneIndex,ANY-method}
\alias{Subset2,ANY,noneIndex,noneIndex-method}
\alias{Subset2,ANY,NAIndex,NAIndex-method}
\alias{Subset2,ANY,NAIndex,ANY-method}
\alias{Subset2,ANY,NAIndex,allIndex-method}
\alias{Subset2,ANY,positiveIndex,positiveIndex-method}
\alias{Subset2,ANY,positiveIndex,ANY-method}
\alias{Subset2,ANY,sliceIndex,sliceIndex-method}
\alias{Subset2,ANY,sliceIndex,ANY-method}
\title{ Generics Subset and Subset2 in package `externalVector' and
  their methods }
\description{
  The generics Subset and Subset2 and their methods are used internally
  and are not for general use. They may be replaced by C code in future.
}
\keyword{methods}

\eof
\name{allIndex-class}
\docType{class}
\alias{allIndex-class}
\alias{Arith,numeric,allIndex-method}
\alias{Arith,allIndex,numeric-method}
\alias{[,allIndex,ANY,missing,missing-method}
\alias{[,allIndex,missing,missing,missing-method}
\alias{[<-,allIndex,ANY,ANY,ANY-method}
\alias{[[<-,allIndex-method}
\alias{allNA,allIndex-method}
\alias{anyNA,allIndex-method}
\alias{initialize,allIndex-method}
\alias{length,allIndex-method}
\alias{length<-,allIndex-method}
\alias{dim<-,allIndex,ANY-method}

\title{Class "allIndex", index class representing a subscript for all
  elements of a vector or an array dimension }
\description{ When used as a subscript for a vector object \code{x},
  this is equivalent to \code{seq(length(x))}. When used as a subscript for
  the \code{i}-th dimension of an array object \code{x}, this is
  equivalent to \code{seq(dim(x)[i])}. }
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("allIndex")}.
Objects from this class can not be modified and they are all equivalent.
}
\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"logical"}, this is
      always equal to \code{TRUE}. }
  }
}
\section{Extends}{
Class \code{"logical"}, directly.
Class \code{"vectorIndex"}, directly.
}
\section{Methods}{
  Signature components for the methods are:

  \tabular{ll}{
    x \tab The class "allIndex"\cr
    scalar \tab Length one positive "integer"\cr
    i \tab The class "ANY"\cr
    value \tab The class "ANY"\cr
    .Object \tab The class "allIndex"\cr
  }

  \describe{
    \item{\code{x+scalar}:}{ Add \code{scalar} to \code{x}.
      Result is \code{x}. }
    \item{\code{scalar+x}:}{ Add \code{scalar} to \code{x}.
      Result is \code{x}. }
    \item{\code{x*scalar}:}{ Multiply \code{x} by \code{scalar}.
      Result is \code{x}. }
    \item{\code{scalar*x}:}{ Multiply \code{x} by \code{scalar}.
      Result is \code{x}. }
    \item{dim(x) <- value}{ Set a dimension on \code{x}. If value is of
      length 2, this may create a "matrixIndex". }
    \item{\code{x[i]}:}{ Subset x. Result is i. }
    \item{\code{x[]}:}{ Subset x with the subscript missing. Result is x. }
    \item{\code{x[i] <- value}:}{ Subassign x. Results in an error (as
      objects from class \code{"allIndex"} are not mutable. }
    \item{\code{x[[i]] <- value}:}{ Subassign x. Results in an error (as
      objects from class \code{"allIndex"} are not mutable. }
    \item{\code{allNA(x)}:}{ Are all elements of \code{x} \code{NA}? Result is
      always \code{FALSE}. }
    \item{\code{anyNA(x)}:}{ Is any element of \code{x} \code{NA}? Result is
      always \code{FALSE}. }
    \item{\code{initialize(.Object)}:}{Initialize an object from class
      \code{"allIndex"}. }
    \item{\code{length(x)}:}{ Return the length of \code{x} which is always
      \code{Inf}. }
    \item{\code{length(x) <- value}:}{ Modify the length of \code{x}. Always
      results in an error. }
  }
}
\seealso{
  \code{\link{vectorIndex-class}} for the super class of \code{"allIndex"}.
}
\keyword{classes}
\keyword{methods}

\eof
\name{allNA}
\alias{allNA}
\alias{anyNA}
\alias{allNA,ANY-method}
\alias{anyNA,ANY-method}
\title{ The generics "allNA" and "anyNA". }
\description{
  The two generic \code{"allNA"} returns \code{TRUE} if all of its
  arguments are NA's and \code{FALSE} otherwise.

  The two generic \code{"anyNA"} returns \code{TRUE} if any of its
  arguments is NA and \code{FALSE} otherwise.
}
\usage{
allNA(x)
anyNA(x)
}
\arguments{
  \item{x}{ An object on which \code{is.na} can be applied. }
}
\value{
  Either \code{TRUE} or \code{FALSE}.
}
\examples{
allNA(1:3)
allNA(c(NA, NA, NA))
allNA(c(1, 2, NA))
anyNA(1:3)
anyNA(c(NA, NA, NA))
anyNA(c(1, 2, NA))
}
\keyword{methods}

\eof
\name{allocators}
\alias{allocatedSize}
\alias{initializeResource}
\alias{getPointer}
\alias{allocator}
\alias{allocate}
\alias{allocatedType}
\alias{deallocate}
\alias{external.size}
\alias{external.size<-}
\alias{reinitializePointer}
\title{ Generics associated with external resources and their allocators}
\description{
  These are generics used with objects of class "externalResource" and
  "externalAllocator".
}
\usage{
allocator(resource)
getPointer(resource)
initializeResource(resource, ptr, size, type, \dots)
allocatedSize(resource)
allocatedType(resource)
allocate(resource, alloc, size, type, \dots)
deallocate(resource, alloc)
external.size(resource, alloc)
external.size(resource, copy, alloc) <- value
reinitializePointer(resource, alloc)
}
\arguments{
  \item{resource}{ An object which is an "externalResource". }
  \item{alloc}{ An object which is an "externalAllocator". }
  \item{ptr}{ An external pointer. }
  \item{size}{The size - if given for a non-vector type, the numebr of
    bytes. Otherwise, the length of the vector. }
  \item{type}{Object represnting the type stored in the resource.}
  \item{copy}{Logical, if
    \code{TRUE} (the default), then the new memory is initialized to
    the content of the old memory for the minimum of old and new sizes.
    Content of any uninitialized memory is undefined. Under any circumstance,
    the inherent type of the allocated memory remains the same as
    its initial value.
  }
  \item{value}{Integer, new size. }
  \item{\dots}{Further arguments passed to or from other functions.}
}
\value{
  \code{allocator} returns the allocator to be used by default with
  \code{resource}.
  
  \code{getPointer} returns the \code{"externalptr"}
  associated with \code{resource}.
  
  \code{initializeResource}
  intializes \code{resource} with \code{ptr} of type
  \code{"externalptr"} and returns \code{resource}.
      
  \code{allocatedSize} returns the size of memory to be allocated for
  \code{resource}. If \code{allocatedType(resource)} is
  an \R basic vector type, then the size is the length of the
  vector. Otherwise the size is the total number of bytes.
  
  \code{allocatedType} returns an object representing the type to be
  stored in the resource
  
  \code{allocate} allocates the
  external pointer in resource using the allocator \code{alloc} for
  \code{resource}. If \code{type} is a basic vector object, then
  it allocates an object of same mode with length
  \code{size} and otherwise allocates \code{size} bytes of raw memory.
  The \code{resource} object is initialized by a call to
  \code{initializeResource}.

  \code{deallocate} asks the allocator \code{alloc} to
    deallocate the memory in \code{resource}. The result is
    allocator dependant.
  
  \code{external.size} returns the argument \code{size} used in the
  last call to \code{allocate} for \code{resource}.
  
  The replacement form can be used to change the size of the allocated
  memory. If \code{value} is same as \code{external.size(resource)}, then no
  action is taken. Otherwise, this reallocates the memory in
  \code{resource} using the allocator \code{alloc} (or
  \code{allocator(resource)} if \code{alloc} is missing) with new size
  \code{value} and the same type as earlier.

  \code{reinitializePointer} tries to reinitialize the memory pointer
  in resource after \code{resource} was saved and restored as an \R image
  (by serialization code, by saving the \R workspace, or by an explicit call to
  \code{save}).
}
\seealso{ \code{\link{externalAllocator-class}},
  \code{\link{externalResource-class}} }
\keyword{methods}

\eof
\name{as.Rvector}
\alias{as.Rvector}
\alias{as.Rmatrix}
\alias{as.Rvector,array-method}
\alias{as.Rvector,matrix-method}
\alias{as.Rvector,logical-method}
\alias{as.Rvector,integer-method}
\alias{as.Rvector,numeric-method}
\alias{as.Rvector,complex-method}
\alias{as.Rvector,character-method}
\alias{as.Rvector,list-method}
\alias{as.Rmatrix,matrix-method}
\alias{as.Rmatrix,ANY-method}
\title{ convert an object to an R basic vector or matrix}
\description{
  \code{as.Rvector}, a generic, attempts to coerce its argument into a
  basic R vector of a convenient mode. All attributes of \code{x} are
  preserved.
  
  Default methods are provided for basic R objects which are returned
  unchanged by as.Rvector.
  
  \code{as.Rmatrix}, a generic, first converts its argument to a basic R
  vector and then applies as.matrix to the result.
}
\usage{
as.Rvector(x)
as.Rmatrix(x)
}
\arguments{
  \item{x}{ An object }
}
\details{
  This preserves all the non-slot attributes of the object \code{x}
  when converting to a basic R vector. In addition, if any of
  \code{dim(x)}, \code{dimnames(x)}, \code{names(x)} is not
  \code{NULL}, the corresponding attribute in the result is also set.
}
\value{
  The value from as.Rvector is a basic R vector of mode "logical", "integer",
  "numeric", "complex", "character" or "list".
  The value from as.Rmatrix is a basic R matrix.
}
\seealso{ \code{\link{as.vector}} for converting an object to an
  appropriate vector with all attributes removed and
  \code{\link{as.matrix}} for converting a vector like object to a
  matrix.}
\examples{
x <- 1:2
names(x) <- letters[1:2]
as.vector(x)
as.Rvector(x)
as.Rmatrix(x)
}
\keyword{classes}

\eof
\name{asEach}
\alias{asEach}
\alias{asEach,ANY,character-method}
\title{ Coerce each element of an external vector to a particular type }
\description{
  This generic coerces each element of an external vector to a
  particular type.
}
\usage{
asEach(x, type, arrayOnly = FALSE)
}
\arguments{
  \item{x}{ An external vector object. }
  \item{type}{ Character-string, a basic vector mode. }
  \item{arrayOnly}{ If arrayOnly is \code{FALSE} (the default) do not
    copy extra slots like \code{Names}, \code{DimNames} or \code{Dim}.
  }
}
\value{
  Another external vector object of same length but with
  \code{class(defaultElement(x)) == class(type)}.
}
\seealso{ \code{\link[methods]{as}} to convert an object to a different class. }
\examples{
x <- externalCharacter(4)
x[] <- c("1", "2", "3", "4")
asEach(x, "integer")
}
\keyword{classes}

\eof
\name{defaultElement}
\alias{defaultElement}
\alias{defaultElement,vector-method}
\title{ Return the default element for a vector object. }
\description{
  For a vector object, this generic returns a length one vector with the
  default element for the vector as the first (and only) element.
}
\usage{
defaultElement(x)
}
\arguments{
  \item{x}{ A vector object. }
}
\value{
  A length one vector with the default element.
}
\seealso{\code{\link{externalVector-class}} for the method for this
  generic for the "externalVector" class. }
\examples{
defaultElement(1:5)
defaultElement(list(1, 2))
}
\keyword{methods}

\eof
\name{exprMatrix-class}
\docType{class}
\alias{exprMatrix-class}

\title{ Class union for objects behaving like a matrix }
\description{  Class union "exprMatrix" represents the type of exprs and
  se.exprs slots in "exprSet" objects. One of the classes in the union
  is always "matrix" }
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}
\section{Methods}{
No methods defined with class "exprMatrix" in the signature.
}
\keyword{classes}

\eof
\name{externalAllocator-class}
\docType{class}
\alias{externalAllocator-class}
\alias{allocate,externalResource,externalAllocator-method}
\alias{deallocate,externalResource,externalAllocator-method}
\alias{external.size,externalResource,externalAllocator-method}
\alias{external.size<-,externalResource,logical,externalAllocator,ANY-method}
\alias{reinitializePointer,externalResource,externalAllocator-method}

\title{Class "externalAllocator", base class for memory allocators for
  external resources }
\description{ Class \code{"externalAllocator"} is a virtual class with no
  slots. It represents memory allocators that allocate raw memory to be
  held in an object of class \code{"externalptr"}. Instead of returning the
  \code{"externalptr"} object directly, the allocators deal with
  subclasses of \code{"externalResource"} that hold an object of
  class \code{"externalptr"}. }
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}
\section{Virtual Methods}{
  Attempt to execute these methods would result in an error unless they
  have been redefined for a subclass of \code{"externalAllocator"}.

  Signature components for implementation of the methods:

  \tabular{ll}{
    resource \tab The class "externalResource"\cr
    alloc \tab A subclass of "externalAllocator"\cr
    size \tab The class "ANY"\cr
    type \tab The class "ANY"\cr
    copy \tab The class "logical"\cr
    value \tab The class "ANY"\cr
  }

  Description of the virtual methods:

  \describe{
      \item{allocate(resource, alloc, size, type, \dots):}{ Allocate the
	external pointer in \code{resource} using the allocator
	\code{alloc}. If \code{type} is a basic vector object, then
	allocate an object of same mode with length
	\code{size}. Otherwise allocate \code{size} bytes of raw
	memory. This method should end with a call to
	\code{initializeResource} to initialize the \code{resource}
        object. }
      \item{deallocate(resource, alloc):}{ The allocator \code{alloc}
	should try to dealloacte the raw memory in \code{resource}. It
	should not modify the \code{resource} object in any way other
	than modifying the object of class \code{"externalptr"} in
	\code{resource} to reflect the deallocation. }
      \item{external.size}{ Return the size of the allocated memory in
	\code{resource}. }
      \item{external.size<-(resource, copy, alloc, value):}{ If
	\code{value} is same as \code{external.size(resource)}, then no
	action is taken. Otherwise, reallocate the memory in
	\code{resource} using the allocator \code{alloc} with new size
	\code{value}. If \code{copy} is \code{TRUE} (the
	default), then the new memory is initialized to the content of
	the old memory for the minimum of old and new sizes. Content of
	any uninitialized memory is undefined. }
      \item{reinitializePointer(resource, alloc):}{ If the object
	\code{resource} was saved as an \R image (by serialization
	code, by saving the \R workspace, or by an explicit call to
	\code{save}) then the raw memory pointer in any
	\code{"externalptr"} object in it would be set to \code{0}. This
	method tries to reinitialize the raw memory pointer. The exact
	result is allocator dependant. }
  }
}
\author{ Saikat DebRoy \email{<saikat@stat.wisc.edu>} }
\seealso{
  \code{\link{externalResource-class}} for more on how to
  use an allocator with objects from subclasses of
  \code{"externalResource"}.

  \code{\link{gcAllocator-class}} for an example of a
  subclass of \code{"externalAllocator"}.

  \code{\link{setVirtualMethod}} for more on virtual methods.
}
\keyword{classes}
\keyword{methods}

\eof
\name{externalMatrix}
\alias{externalMatrix}
\title{ Create an external matrix object }
\description{
  This function can be used to create a new external matrix object.
}
\usage{
externalMatrix(data = NA, nrow = 1, ncol = 1, byrow = FALSE, dimnames =
NULL, storageClass = (if (is(data, "externalVectorWithStorage"))
class(data@storage) else "simpleStorage"), indirect = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{ An optional data vector. }
  \item{nrow}{ The desired number of rows. }
  \item{ncol}{ The desired number of columns. }
  \item{byrow}{ Not used. }
%  \item{byrow}{ Logical. If 'FALSE' (the default) the matrix is filled by
%          columns, otherwise the matrix is filled by rows. }
  \item{dimnames}{ A \code{dimnames} value for the external matrix: a
    \code{list} of length 2. }
  \item{storageClass}{ name of the subclass of "externalStorage" to be
    used for storing the external vector elements. }
  \item{indirect}{ If \code{TRUE}, return an
    "indirectExternalVector" object. }
}
\details{
  This function is similar to the \code{matrix} function in \R base package.
}
\value{
  If \code{indirect} is \code{FALSE} (the default)
  a new external matrix object with correct dimension and dimnames.
  Otherwise create a new external vector object but return it by
  wrapping it in an "indirectExternalMatrix" object with the correct
  dimension and dimnames.
}
\seealso{ \code{\link{matrix}} }
\examples{
x <- externalMatrix(1:6, nrow=2, ncol=3)
dim(x)
x[1, 1:2] # drop = FALSE by default
log(x)
}
\keyword{ array }

\eof
\name{externalResource-class}
\docType{class}
\alias{externalResource-class}
\alias{allocate,externalResource,missing-method}
\alias{allocatedSize,externalResource-method}
\alias{allocatedType,externalResource-method}
\alias{allocator,externalResource-method}
\alias{deallocate,externalResource,missing-method}
\alias{external.size,externalResource,missing-method}
\alias{external.size<-,externalResource,ANY,missing,ANY-method}
\alias{getPointer,externalResource-method}
\alias{initialize,externalResource-method}
\alias{initializeResource,externalResource,externalptr-method}
\alias{reinitializePointer,externalResource,missing-method}

\title{Class "externalResource", base class for external resource
  allocated by an externalAllocator }
\description{ Class \code{"externalResource"} is a virtual class with no
  slots. External allocators, represented by subclasses of class
  \code{"externalAllocator"} can only allocate external pointers
  contained in objects from a subclass of \code{"externalResource"}. }
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}
\section{Methods for the virtual class "externalResource"}{
  These methods are defined for the virtual class "externalResource" and
  need not be redefined by its subclasses.

  Signature components for the methods are:

  \tabular{ll}{
    resource \tab The class "externalResource"\cr
    alloc \tab The class "missing"\cr
    ptr \tab The class "externalptr"\cr
    size \tab The class "ANY"\cr
    type \tab The class "ANY"\cr
    copy \tab The class "logical"\cr
    value \tab The class "ANY"\cr
  }

  The argument \code{alloc} has the class "missing" wherever it appears
  in these methods. That means, to invoke these methods, the argument
  \code{alloc} must be ommitted from the call and to use any of the
  additional arguments, they must be specfied by name.
  For example, use \code{allocate(resource, size=size, type=type)}
  and not \code{allocate(resource, size, type)}.

  Description of the methods:
  
  \describe{
      \item{allocatedType(resource):}{ Returns \code{NULL}. }
      \item{allocate(resource, alloc, size, type, \dots):}{ Allocate the
	external pointer in resource using the default allocator for
	\code{resource}. If \code{type} is a basic vector object, then
	allocate an object of same mode with length
	\code{size}. Otherwise allocate \code{size} bytes of raw memory.
	The \code{resource} object is initialized by a call to
	\code{initializeResource}. }
      \item{deallocate(resource, alloc):}{ Ask the default allocator to
	deallocate the memory in \code{resource}. The result is
	allocator dependant. }
      \item{external.size}{ Return the argument \code{size} used in the
	last call to \code{allocate} for \code{resource} }
      \item{external.size<-(resource, copy, alloc, value):}{ If
	\code{value} is same as \code{external.size(resource)}, then no
	action is taken. Otherwise, reallocate the memory in
	\code{resource} using the default allocator with new size
	\code{value} and the same type as earlier. If \code{copy} is
	\code{TRUE} (the default), then the new memory is initialized to
	the content of 
	the old memory for the minimum of old and new sizes. Content of
	any uninitialized memory is undefined. Under any circumstance,
	the inherent type of the allocated memory remains the same as
	its initial value. }
      \item{initialize(.Object):}{ Code called by
	new("resourceSubclass", \dots) if "resourceSubclass" is a subclass
	of "externalResource" and either has no \code{initialize} method
	of its own or its \code{initialize} method has
	\code{callNextMethod()} in its body. Returns the result of
	\code{allocate(.Object, \dots)}}
      \item{reinitializePointer(resource, alloc):}{ If the object
	\code{resource} was saved as an \R image (by serialization
	code, by saving the \R workspace, or by an explicit call to
	\code{save}) then the raw memory pointer in any
	\code{"externalptr"} object in it would be set to \code{0}. This
	method tries to reinitialize the raw memory pointer. The exact
	result is allocator dependant. }
  }
}
\section{Virtual Methods}{
  Attempt to execute these methods would result in an error unless they
  have been redefined for a subclass of \code{"externalResource"}.

  Signature components for implementation of the methods:

  \tabular{ll}{
    resource \tab A subclass of "externalResource"\cr
    ptr \tab The class "externalptr"\cr
    size \tab The class "ANY"\cr
    type \tab The class "ANY"\cr
  }

  Description of the virtual methods:
  
  \describe{
      \item{allocatedSize(resource):}{ Size of memory to be allocated for
	\code{resource}. If \code{allocatedType(resource)} is
	an \R basic vector type, then the size is the length of the
	vector. Otherwise the size is the total number of bytes. }
      \item{allocator(resource):}{ The allocator to be used by default with
	this \code{resource}. }
      \item{getPointer(reource):}{ Return the \code{"externalptr"}
	associated with this \code{resource}. }
      \item{initializeResource(resource, ptr, size, type, \dots):}{
	Intialize \code{resource} with \code{ptr} of type
	\code{"externalptr"}. The \code{size} and \code{type} arguments are
	identical to that obtained from previous calls to
	\code{allocatedSize(resource)} and \code{allocatedType(resource)}. }
    }
}
\section{Other Virtual Methods}{
  These methods must be redefined for subclasses for
  \code{"externalAllocator"}. It is not necessary to define them for
  specific subclasses of \code{"externalResource"}.

  Signature components for implementation of the methods:

  \tabular{ll}{
    resource \tab The class "externalResource"\cr
    alloc \tab A subclass of "externalAllocator"\cr
    size \tab The class "ANY"\cr
    type \tab The class "ANY"\cr
    copy \tab The class "logical"\cr
    value \tab The class "ANY"\cr
  }

  Description of the virtual methods:

  \describe{
      \item{allocate(resource, alloc, size, type, \dots):}{ Allocate the
	external pointer in \code{resource} using the allocator
	\code{alloc}. }
      \item{deallocate(resource, alloc):}{ Ask the allocator \code{alloc}
	to deallocate the memory in \code{resource}. }
      \item{external.size}{ Return the size of the allocated memory in
	\code{resource}. }
      \item{external.size<-(resource, copy, alloc, value):}{ Reallocate
	the memory in \code{resource} using the allocator \code{alloc}. }
      \item{reinitializePointer(resource, alloc):}{ If the raw memory
	pointer in \code{resource} is zero, try to reinitialize it. }
  }
}
\author{ Saikat DebRoy \email{<saikat@stat.wisc.edu>} }
\seealso{
  \code{\link{externalAllocator-class}} for more details on how to
  use an allocator with objects from subclasses of
  \code{"externalResource"}.

  \code{\link{gcAllocator-class}} for an example of a simple
  subclass of \code{"externalResource"}.

  \code{\link{setVirtualMethod}} for more on virtual methods.
}
\keyword{classes}
\keyword{methods}

\eof
\name{externalStorage-class}
\docType{class}
\alias{externalStorage-class}
\alias{allocator,externalStorage-method}
\alias{getNativeStorageMethods,externalStorage-method}
\alias{getPointer,externalStorage-method}
\alias{initialize,externalStorage-method}
\alias{initializeResource,externalStorage-method}
\alias{internalType,externalStorage-method}

\title{Class "externalStorage", base class for external storage for
  objects of class "externalVectorWithStorage" }
\description{ The \code{"externalStorage"} class represents the base
  class of the storage backend implementation used by the class
  \code{"externalVectorWithSToraghe"}. A pointer to the external storage
  allocated or accessed by an object of class \code{"externalStorage"}
  is kept in an external pointer. }
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}
\section{Slots}{
  \describe{
    \item{\code{ptr}:}{Object of class \code{"externalptr"}, keeps the
      pointer to the external storage. }
    \item{\code{type}:}{Object of class \code{"vector"}, a vector object
      of length one representing the type of object stored in the
      external storage. Usually one of the basic R vector types. }
    \item{\code{length}:}{Object of class \code{"integer"}, Cached value
      for the length of the vector stored in the external storage. }
  }
}
\section{Extends}{
Class \code{"externalResource"}, directly.
}
\section{Methods}{
  Signature components for the methods:

  \tabular{ll}{
    resource \tab The class "externalStorage"\cr
    x \tab The class "externalStorage"\cr
    ptr \tab The class "externalptr"\cr
    size \tab The class "ANY"\cr
    type \tab The class "ANY"\cr
  }

  \describe{
      \item{allocator(resource):}{ The allocator to be used by default with
	this \code{resource}. Returns an object of class
	\code{"gcAllocator"}. }
      \item{getPointer(reource):}{ Return the \code{ptr} slot. }
      \item{initializeResource(resource, ptr, size, type, \dots):}{
	Set the \code{ptr} slot of resource to \code{ptr}. }
      \item{initialize(.Object, type = logical(1), length =
	as.integer(0)):}{ This checks the validity of \code{type} and
	\code{length} arguments and coerce the \code{length} argument to
	class \code{"integer"} before using them to set the
	corresponding slots. Finally a call to \code{callNextMethod} is
	made to use the initializer for class \code{"externalResource"}
	which, via a call to the \code{initializeResource} method above
	sets \code{ptr} slot. }
      \item{internalType(x):}{ The type of object to be stored
	within the resource. It is an \R basic vector type
	(\code{"logical"}, \code{"integer"}, \code{"numeric"},
	\code{"complex"}, \code{"character"} or \code{"list"}). }
      \item{getNativeStorageMethods(x):}{ Get the \code{"externalptr"}
	stored in the \code{"nativeStorageMethods"} object associated
	with a particular subclass of \code{"externalStorage"} and a
	basic R vector type. }
  }
}
\seealso{
  \code{\link{setExternalStorageClass}} for how to set a subclass of
  \code{"externalStorage"}.
  
  \code{\link{nativeStorageMethods-class}} for the C structure
  containing C function pointers for a particular storage methods class.
  
  \code{\link{externalVectorWithStorage-class}} for how the
  \code{"externalStorage"} class is used.
  
  \code{\link{simpleStorage-class}} for a simple implementation of
  \code{"externalStorage"}.
}
\keyword{classes}

\eof
\name{externalVector-class}
\docType{class}
\alias{externalVector-class}
\alias{asEach,externalVector,character-method}
\alias{c,externalVector-method}
\alias{apply,externalVector-method}
\alias{as.complex,externalVector-method}
\alias{as.data.frame,externalVector-method}
\alias{as.double,externalVector-method}
\alias{as.integer,externalVector-method}
\alias{as.list,externalVector-method}
\alias{as.logical,externalVector-method}
\alias{as.matrix,externalVector-method}
\alias{as.numeric,externalVector-method}
\alias{as.single,externalVector-method}
\alias{coerce,externalVector,vector-method}
\alias{coerce,externalVector,matrix-method}
\alias{coerce,vector,externalVector-method}
\alias{coerce,matrix,externalVector-method}
\alias{defaultElement,externalVector-method}
\alias{dimnames<-,externalVector,NULL-method}
\alias{dimnames<-,externalVector,list-method}
\alias{names<-,externalVector,externalVector-method}
\alias{names<-,externalVector,ANY-method}
\alias{names<-,externalVector,NULL-method}
\alias{setDimNames,externalVector-method}
\alias{setNames,externalVector-method}
\alias{show,externalVector-method}
\alias{as.character,externalVector-method}
\alias{as.Rvector,externalVector-method}
\alias{as.vector,externalVector-method}
\alias{lapply,externalVector-method}
\alias{sapply,externalVector-method}
\alias{mean,externalVector-method}
\alias{median,externalVector,ANY-method}
\alias{median,externalVector,missing-method}
\alias{cor,ANY,externalVector-method}
\alias{cov,ANY,externalVector-method}
\alias{var,ANY,externalVector-method}
\alias{cor,externalVector,ANY-method}
\alias{cov,externalVector,ANY-method}
\alias{var,externalVector,ANY-method}
\alias{cov2cor,externalVector-method}

\title{Class "externalVector", base class for vector objects stored in
  an external resource }
\description{ This class represents objects that behave like R basic
  vectors but are stored in some external resource. }
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}
\section{Methods}{
  Common signature components for the methods:

  \tabular{ll}{
    x \tab The class "externalVector"\cr
    X \tab The class "externalVector"\cr
  }

  Description of the methods:
  
  \describe{
    \item{asEach(x, type, arrayOnly)}{ Coerce \code{x} so that it now
      behaves like a basic R vector with the same class as \code{type}.}
    \item{c(x, \dots, recursive)}{ Combine all the arguments into a
      single vector. }
      behaves like a basic R vector with the same class as \code{type}.}
    \item{as.Rvector(x)}{ Coerce \code{x} to a basic R vector with same
      names, dimension and dimnames (if any of these are present in
      \code{x}). }
    \item{as.vector(x, mode)}{ Return an external vector which behaves
      like a basic R vector of mode \code{mode}. }
    \item{as.logical(x, \dots)}{ Return an external vector with all
      elements coerced to logical. }
    \item{as.integer(x, \dots)}{ Return an external vector with all
      elements coerced to integer. }
    \item{as.double(x, \dots)}{ Return an external vector with all
      elements coerced to numeric. }
    \item{as.numeric(x, \dots)}{ Return an external vector with all
      elements coerced to numeric. }
    \item{as.charcater(x)}{ Return an external vector with all
      elements coerced to character. }
    \item{as.list(x, \dots)}{ Return an external vector that behaves like
      a list. }
    \item{as.single(x, \dots)}{ Return an external vector with all
      elements coerced to numeric and a "CSingle" attribute set to
      \code{TRUE}. }
    \item{as.data.frame(x, row.names, optional}{ Create a data.frame
      object from \code{x}. }
    \item{as.matrix(x)}{ If \code{length(dim(x))} is \code{2}, return
      \code{x} unchanged. Otherwise, return \code{x} as a matrix with a
      single column. }
    \item{defaultElement(x)}{ The default element of \code{x} contained
      in a basic R vector. }
    \item{dimnames(x) <- value}{ Set dimnames of \code{x} to value. }
    \item{names(x) <- value}{ Set names of \code{x} to value. }
    \item{show(object)}{ Display \code{object}. }
    \item{apply(X, MARGIN, FUN, \dots}{ Coerce X to an R matrix and call
      the default \code{apply} method. }
    \item{lapply(X, FUN, \dots)}{ Lapply for "externalVector"
      class. }
    \item{sapply(X, FUN, \dots, simplify, USE.NAMES)}{ Sapply for
      "externalVector" class. }
    \item{mean(x, trim = 0, na.rm = FALSE, \dots)}{ Mean and trimmed
      mean of \code{x}. }
    \item{median(x, na.rm = FALSE)}{ Mean of \code{x}. }
  }
}
\section{Virtual Methods}{
  Common signature components for implementation of the virtual methods:

  \tabular{ll}{
    x \tab A subclass of "externalVector"\cr
    value \tab The class "ANY"\cr
  }

  Description of the virtual methods:
  
  \describe{
    \item{setDimNames(x, value)}{ Set the dimnames of \code{x} to
      \code{value} without any error checking or coercion. Return the
      modified \code{x}. }
    \item{setNames(x, value)}{ Set the names of \code{x} to \code{value}
      without any error checking or coercion. Return the modified
      \code{x}. }
  }
}
\seealso{
  \code{\link{externalVectorWithStorage-class}},\code{\link{matrixIndex-class}},
  \code{\link{indirectExternalVector-class}},
  \code{\link{indirectExternalMatrix-class}}, for subclasses of
  \code{"externalVector"}.
}
\keyword{classes}

\eof
\name{externalVector}
\alias{externalVector}
\alias{externalLogical}
\alias{externalInteger}
\alias{externalNumeric}
\alias{externalComplex}
\alias{externalCharacter}
\alias{externalList}
\title{ Creating external vectors }
\description{
  These functions can be used to create new external vector objects
  with newly allocated storage.
}
\usage{
externalVector(type = "logical", length = 0, storageClass = "simpleStorage", indirect = FALSE)
externalLogical(length = 0, storageClass = "simpleStorage", indirect = FALSE)
externalInteger(length = 0, storageClass = "simpleStorage", indirect = FALSE)
externalNumeric(length = 0, storageClass = "simpleStorage", indirect = FALSE)
externalComplex(length = 0, storageClass = "simpleStorage", indirect = FALSE)
externalCharacter(length = 0, storageClass = "simpleStorage", indirect = FALSE)
}
\arguments{
  \item{type}{ A character-string, the name of the class of basic R
    vectors to be represented by the new external vector object. }
  \item{length}{ Length of the new external vector }
  \item{storageClass}{ name of the subclass of "externalStorage" to be
    used for storing the external vector elements. }
  \item{indirect}{ If \code{TRUE}, return an
    "indirectExternalVector" object. }
}
\details{
  These functions are similar to the \code{vector}, \code{logical},
  \code{integer}, \code{numeric}, \code{complex} and \code{character}
  functions in the \R base package. These functions have an extra
  argument \code{storageClass} to specify the external storage class to
  use. The default is to use "simpleStorage" which uses garbage
  collected memory from \R for storage.
}
\value{
  If \code{indirect} is \code{FALSE} (the default) the
  a new external vector object of the given length, given type with
  elements stored in an external storage object with class \code{storageClass}.
  Otherwise create the new external vector object but return it by
  wrapping it in an "indirectExternalVector" object.
}
\seealso{ \code{\link{vector}}, \code{\link{logical}},
  \code{\link{integer}}, \code{\link{numeric}}, \code{\link{complex}},
  \code{\link{character}},  \code{\link{list}}
}
\examples{
x <- externalVector("numeric", 4)
x[] <- 1:4
x[1:3]
y <- x+2
y
x+y
}
\keyword{classes}

\eof
\name{externalVectorWithStorage-class}
\docType{class}
\alias{externalVectorWithStorage-class}
\alias{Arith,ANY,externalVectorWithStorage-method}
\alias{Arith,externalVectorWithStorage,ANY-method}
\alias{Arith,externalVectorWithStorage,externalVectorWithStorage-method}
\alias{Arith,externalVectorWithStorage,missing-method}
\alias{Compare,ANY,externalVectorWithStorage-method}
\alias{Compare,externalVectorWithStorage,ANY-method}
\alias{Compare,externalVectorWithStorage,externalVectorWithStorage-method}
\alias{Complex,externalVectorWithStorage-method}
\alias{Math,externalVectorWithStorage-method}
\alias{Ops,ANY,externalVectorWithStorage-method}
\alias{Ops,externalVectorWithStorage,ANY-method}
\alias{Summary,externalVectorWithStorage,missing-method}
\alias{Summary,externalVectorWithStorage,logical-method}
\alias{[,externalVectorWithStorage,ANY,ANY,ANY-method}
\alias{[<-,externalVectorWithStorage,ANY,ANY,externalVectorWithStorage-method}
\alias{[<-,externalVectorWithStorage,ANY,ANY,ANY-method}
\alias{as.Rvector,externalVectorWithStorage-method}
\alias{asEach,externalVectorWithStorage,character-method}
\alias{colMeans,externalVectorWithStorage,missing,missing-method}
\alias{colMeans,externalVectorWithStorage,missing,ANY-method}
\alias{colMeans,externalVectorWithStorage,ANY,missing-method}
\alias{colMeans,externalVectorWithStorage,logical,numeric-method}
\alias{colSums,externalVectorWithStorage,missing,missing-method}
\alias{colSums,externalVectorWithStorage,missing,ANY-method}
\alias{colSums,externalVectorWithStorage,ANY,missing-method}
\alias{colSums,externalVectorWithStorage,logical,numeric-method}
\alias{dim,externalVectorWithStorage-method}
\alias{dim<-,externalVectorWithStorage,NULL-method}
\alias{dim<-,externalVectorWithStorage,ANY-method}
\alias{dimnames,externalVectorWithStorage-method}
\alias{getExternalStorage,externalVectorWithStorage-method}
\alias{getNativeStorageMethods,externalVectorWithStorage-method}
\alias{initialize,externalVectorWithStorage-method}
\alias{is.finite,externalVectorWithStorage-method}
\alias{is.infinite,externalVectorWithStorage-method}
\alias{is.na,externalVectorWithStorage-method}
\alias{is.nan,externalVectorWithStorage-method}
\alias{length,externalVectorWithStorage-method}
\alias{length<-,externalVectorWithStorage-method}
\alias{log,externalVectorWithStorage,missing-method}
\alias{log,externalVectorWithStorage,ANY-method}
\alias{log,externalVectorWithStorage,externalVectorWithStorage-method}
\alias{match,ANY,externalVectorWithStorage-method}
\alias{match,externalVectorWithStorage,ANY-method}
\alias{names,externalVectorWithStorage-method}
\alias{names<-,externalVectorWithStorage,NULL-method}
\alias{rebind,externalVectorWithStorage-method}
\alias{rowMeans,externalVectorWithStorage,missing,missing-method}
\alias{rowMeans,externalVectorWithStorage,missing,ANY-method}
\alias{rowMeans,externalVectorWithStorage,ANY,missing-method}
\alias{rowMeans,externalVectorWithStorage,logical,numeric-method}
\alias{rowSums,externalVectorWithStorage,missing,missing-method}
\alias{rowSums,externalVectorWithStorage,missing,ANY-method}
\alias{rowSums,externalVectorWithStorage,ANY,missing-method}
\alias{rowSums,externalVectorWithStorage,logical,numeric-method}
\alias{setDimNames,externalVectorWithStorage-method}
\alias{setNames,externalVectorWithStorage-method}
\alias{all,externalVectorWithStorage-method}
\alias{any,externalVectorWithStorage-method}
\alias{max,externalVectorWithStorage-method}
\alias{min,externalVectorWithStorage-method}
\alias{range,externalVectorWithStorage-method}
\alias{prod,externalVectorWithStorage-method}
\alias{sum,externalVectorWithStorage-method}
\alias{lapply,externalVectorWithStorage-method}

\title{Class "externalVectorWithStorage", external vector class with
  an external storage for the vector }
\description{ This is a subclass of \code{"externalVector"} that
  implements the use of arbitrary external backends for storing the
  elements of the vector. }
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{externalVector(type,
  lengtyh, storageClass)}.
}
\section{Slots}{
  \describe{
    \item{\code{storage}:}{Object of class \code{"externalStorage"}, the
    storage backend for the vector elements. }
    \item{\code{Names}:}{Object of class \code{"vectorNamesType"}, names
      of the elements. }
    \item{\code{Dim}:}{Object of class \code{"integerOrNull"}, dimension
      of the object. }
    \item{\code{DimNames}:}{Object of class \code{"listOrNull"},
      dimnames of the object }
  }
}
\section{Extends}{
Class \code{"externalVector"}, directly.
Class \code{"vectorNamesType"}, directly.
}
\section{Methods}{
  \describe{
    \item{e1 op e2}{ For any arithmetic or comparison operation \code{op}
      this returns the result of the operation. Either (or both) of
      \code{e1} and \code{e2} are of class
      \code{"externalVectorWithSTorage"}. }
    \item{Complex(z)}{ Return the result of an operation in the
      \code{Complex} group generic. }
    \item{Math(x)}{ Return the result of an operation in the
      \code{Math} group generic. }
    \item{log(x)}{ Return natural logarithm of \code{x}. }
    \item{log(x, base)}{ Return logarithm of \code{x} with base
      \code{base}. }
    \item{x[i, j, \dots, drop=FALSE]}{ Extract a subset of \code{x}
      according to the given subscripts. }
    \item{x[i, j, \dots] <- value}{ Subassign of \code{x} by \code{value}
      according to the given subscripts. }
    \item{as.Rvector(x)}{ Coerce \code{x} to a basic R vector with same
      names, dimension and dimnames (if any of these are present in
      \code{x}). }
    \item{asEach(x, type, arrayOnly)}{ Coerce \code{x} so that it now
      behaves like a basic R vector with the same class as \code{type}.}
    \item{colSums(x, na.rm, dims)}{ Return the column sums of
      \code{x}. }
    \item{colMeans(x, na.rm, dims)}{ Return the column means of
      \code{x}. }
    \item{rowSums(x, na.rm, dims)}{ Return the row sums of
      \code{x}. }
    \item{rowMeans(x, na.rm, dims)}{ Return the row means of
      \code{x}. }
    \item{dim(x)}{ Return the dimension of \code{x}. }
    \item{dim(x) <- value}{ Set the dimension of \code{x} to \code{value}. }
    \item{dimnames(x)}{ Return the dimnames of \code{x}. }
    \item{getExternalStorage(x)}{ Return the \code{storage} slot of
      \code{x}. }
    \item{getNativeStorageMethods}{ Return
      \code{getNativeStorageMethods(x@storage)}. }
    \item{is.finite(x)}{ Return an external vector of logicals with same
      length as \code{x} indicating if an element of \code{x} is finite. }
    \item{is.infinite(x)}{ Return an external vector of logicals with same
      length as \code{x} indicating if an element of \code{x} is infinite. }
    \item{is.na(x)}{ Return an external vector of logicals with same
      length as \code{x} indicating if an element of \code{x} is \code{NA}. }
    \item{is.nan(x)}{ Return an external vector of logicals with same
      length as \code{x} indicating if an element of \code{x} is \code{NaN}. }
    \item{length(x)}{ Return the length of \code{x}. }
    \item{length(x) <- value}{ Set the length of \code{x} to \code{value}. }
    \item{names(x)}{ Return the names of \code{x}. }
    \item{names(x) <- value}{ Set the names of \code{x} to
      \code{value}. }
    \item{all(x, \dots, na.rm)}{ Are all the values \code{TRUE}? }
    \item{any(x, \dots, na.rm)}{ Are any of the values \code{TRUE}? }
    \item{min(x, \dots, na.rm)}{ Minima of the values. }
    \item{max(x, \dots, na.rm)}{ Maxima of the values. }
    \item{range(x, \dots, na.rm)}{ Range of the values. }
    \item{prod(x, \dots, na.rm)}{ Product of the values. }
    \item{sum(x, \dots, na.rm)}{ Sum of the values. }
    \item{lapply(X, FUN, \dots)}{ Lapply for "externalVectorWithStorage"
      class. }
    \item{setDimNames(x, value)}{ Set the dimnames of \code{x} to
      \code{value} without any error checking or coercion. Return the
      modified \code{x}. }
    \item{setNames(x, value)}{ Set the names of \code{x} to \code{value}
      without any error checking or coercion. Return the modified
      \code{x}. }
    \item{rebind(x, length.x = length(x), type = defaultElement(x),
                 names.x = names(x), dim.x = dim(x), dimnames.x =
		 dimnames(x), \dots, arrayOnly = FALSE)}{ Create an
		 \code{"externalVectorWithStorage"} object with the same
		 external storage backend class as \code{x} but with
		 (possibly) new length, type, names, dim or dimnames. If
		 arrayOnly is FALSE, the names, dim and dimnames for the
		 new object are not set. }
    \item{initialize(.Object, \dots)}{ Initialize a new object from class
      \code{"externalVectorWithSTorage"}. }
  }
}
\seealso{
  \code{\link{externalVector}} and \code{\link{externalMatrix}} for how
  how to create new objects from this class easily.
}
\keyword{classes}

\eof
\name{gcAllocator-class}
\docType{class}
\alias{gcAllocator-class}
\alias{allocate,externalResource,gcAllocator-method}
\alias{deallocate,externalResource,gcAllocator-method}
\alias{external.size,externalResource,gcAllocator-method}
\alias{external.size<-,externalResource,logical,gcAllocator,numeric-method}
\alias{reinitializePointer,externalResource,gcAllocator-method}

\title{Class "gcAllocator", memory allocator class for external resources }
\description{ A memory allocator class for external resources that
  allocates memory using R memory management system. The allocated
  memory is freed by the R garbage collector when all refernce to it is
  removed. }
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("gcAllocator")}.
}
\section{Extends}{
Class \code{"externalAllocator"}, directly.
}
\section{Methods}{
  Signature components for the methods:

  \tabular{ll}{
    resource \tab The class "externalResource"\cr
    alloc \tab The class of "gcAllocator"\cr
    size \tab The class "ANY"\cr
    type \tab The class "ANY"\cr
    copy \tab The class "logical"\cr
    value \tab The class "ANY"\cr
  }

  Description of the methods:

  \describe{
      \item{allocate(resource, alloc, size, type, \dots):}{ Allocate the
	external pointer in \code{resource}.
	If \code{type} is a basic vector object, then
	allocate an object of same mode with length
	\code{size}. Otherwise allocate \code{size} bytes of raw
	memory. The allocation is done by creating an R basic vector
	with same class as type and storing it in the protected field of
	the newly created external pointer. Ends with a call to
	\code{initializeResource} to initialize the \code{resource}
        object. }
      \item{deallocate(resource, alloc):}{ Replace the protected field
	of \code{resource@ptr} with \code{R_NilValue}. }
      \item{external.size}{ Return the size of the allocated memory in
	\code{resource}. }
      \item{external.size<-(resource, copy, alloc, value):}{ If
	\code{value} is same as \code{external.size(resource)}, then no
	action is taken. Otherwise, reallocate the memory in
	\code{resource} with new size \code{value}. If \code{copy} is
	\code{TRUE} (the default), then the new memory is initialized to
	the content of the old memory for the minimum of old and new
	sizes. Content of any uninitialized memory is undefined. }
      \item{reinitializePointer(resource, alloc):}{ If the object
	\code{resource} was saved as an \R image (by serialization
	code, by saving the \R workspace, or by an explicit call to
	\code{save}) then the raw memory pointer in any
	\code{"externalptr"} object in it would be set to \code{0}. This
	method reinitializes the memory (if possible) by using the
      protected field of the external pointer. }
  }
}
\examples{
library(externalVector)
## set a storage class
setClass("testStorage",
         representation(ptr="externalptr"),
         contains="externalResource")
setMethod("initializeResource", "testStorage",
          function(resource, ptr, size, type, \dots)
       {
           resource@ptr <- ptr
           resource
       })
setMethod("allocator", "testStorage",
          function(resource)
          new("gcAllocator"))
setMethod("getPointer", "testStorage",
          function(resource)
          resource@ptr)
setMethod("allocatedSize", "testStorage",
          function(resource)
          32)
## Now create an object from the class
x <- new("testStorage")
x
external.size(x)
external.size(x) <- 64
x
external.size(x)
deallocate(x)
x
external.size(x)
}
\keyword{classes}

\eof
\name{getExternalStorage}
\alias{getExternalStorage}
\title{ Extract external storage}
\description{
  This generic extracts the "externalStorage" object associated with an
  "externalVector" object.
}
\usage{
getExternalStorage(x)
}
\arguments{
  \item{x}{ An "externalVector" object. }
}
\value{
  An "externalStorage" object.
}
\seealso{ \code{\link{externalStorage-class}},
  \code{\link{externalVectorWithStorage-class}} }
\keyword{classes}

\eof
\name{getNativeStorageMethods}
\alias{getNativeStorageMethods}
\alias{getNativeStorageMethods,ANY-method}
\title{ Return the storage methods associated with an object }
\description{
  For an object from a subclass of "externalStorage" or for an object
  with an associated subclass of "externalStorage", this generic
  returns the \code{"externalptr"} stored in the
  \code{"nativeStorageMethods"} object associated with that subclass of
  "externalStorage".
}
\usage{
getNativeStorageMethods(x)
}
\arguments{
  \item{x}{ An object. }
}
\value{
  An "externalptr" object with a pointer to a C storageMethods structure.
}
\seealso{\code{\link{nativeStorageMethods-class}}}
\keyword{classes}
\keyword{programming}

\eof
\name{indirectExternalMatrix-class}
\docType{class}
\alias{indirectExternalMatrix-class}
\alias{as.Rvector,indirectExternalMatrix-method}
\alias{Arith,ANY,indirectExternalMatrix-method}
\alias{Arith,indirectExternalVector,indirectExternalMatrix-method}
\alias{Arith,indirectExternalMatrix,ANY-method}
\alias{Arith,indirectExternalMatrix,indirectExternalMatrix-method}
\alias{Arith,indirectExternalMatrix,indirectExternalVector-method}
\alias{Compare,ANY,indirectExternalMatrix-method}
\alias{Compare,indirectExternalVector,indirectExternalMatrix-method}
\alias{Compare,indirectExternalMatrix,ANY-method}
\alias{Compare,indirectExternalMatrix,indirectExternalMatrix-method}
\alias{Compare,indirectExternalMatrix,indirectExternalVector-method}
\alias{Complex,indirectExternalMatrix-method}
\alias{Math,indirectExternalMatrix-method}
\alias{Ops,ANY,indirectExternalMatrix-method}
\alias{Ops,indirectExternalVector,indirectExternalMatrix-method}
\alias{Ops,indirectExternalMatrix,ANY-method}
\alias{Ops,indirectExternalMatrix,indirectExternalMatrix-method}
\alias{Ops,indirectExternalMatrix,indirectExternalVector-method}
\alias{Summary,indirectExternalMatrix,ANY-method}
\alias{[,indirectExternalMatrix,missing,missing,missing-method}
\alias{[,indirectExternalMatrix,missing,missing,logical-method}
\alias{[,indirectExternalMatrix,missing,ANY,missing-method}
\alias{[,indirectExternalMatrix,missing,ANY,logical-method}
\alias{[,indirectExternalMatrix,ANY,missing,missing-method}
\alias{[,indirectExternalMatrix,ANY,missing,logical-method}
\alias{[,indirectExternalMatrix,ANY,ANY,missing-method}
\alias{[,indirectExternalMatrix,ANY,ANY,ANY-method}
\alias{[<-,indirectExternalMatrix,missing,missing-method}
\alias{[<-,indirectExternalMatrix,ANY-method}
\alias{allNA,indirectExternalMatrix-method}
\alias{anyNA,indirectExternalMatrix-method}
\alias{as.matrix,indirectExternalMatrix-method}
\alias{as.vector,indirectExternalMatrix,missing-method}
\alias{as.vector,indirectExternalMatrix,ANY-method}
\alias{coerce,externalVector,indirectExternalMatrix-method}
\alias{coerce,vector,indirectExternalMatrix-method}
\alias{coerce,matrix,indirectExternalMatrix-method}
\alias{coerce,indirectExternalMatrix,vector-method}
\alias{coerce,indirectExternalMatrix,matrix-method}
\alias{dim,indirectExternalMatrix-method}
\alias{dim<-,indirectExternalMatrix,NULL-method}
\alias{dim<-,indirectExternalMatrix,ANY-method}
\alias{dimnames,indirectExternalMatrix-method}
\alias{dimnames<-,indirectExternalMatrix-method}
\alias{force,indirectExternalMatrix-method}
\alias{getExternalStorage,indirectExternalMatrix-method}
\alias{is.finite,indirectExternalMatrix-method}
\alias{is.infinite,indirectExternalMatrix-method}
\alias{is.na,indirectExternalMatrix-method}
\alias{is.nan,indirectExternalMatrix-method}
\alias{length,indirectExternalMatrix-method}
\alias{length<-,indirectExternalMatrix-method}
\alias{log,ANY,indirectExternalMatrix-method}
\alias{log,indirectExternalVector,indirectExternalMatrix-method}
\alias{log,indirectExternalMatrix,ANY-method}
\alias{log,indirectExternalMatrix,indirectExternalMatrix-method}
\alias{log,indirectExternalMatrix,indirectExternalVector-method}
\alias{log,indirectExternalMatrix,missing-method}
\alias{names,indirectExternalMatrix-method}
\alias{names<-,indirectExternalMatrix-method}
\alias{all,indirectExternalMatrix-method}
\alias{any,indirectExternalMatrix-method}
\alias{min,indirectExternalMatrix-method}
\alias{max,indirectExternalMatrix-method}
\alias{range,indirectExternalMatrix-method}
\alias{sum,indirectExternalMatrix-method}
\alias{prod,indirectExternalMatrix-method}

\title{Class "indirectExternalMatrix", class for indirect matrix subset of an
  externalVector object. }
\description{ Objects from this class store an object of class
  \code{"externalVector"}  and a matrix subscript for that object and
  behaves as if this is an object created by applying that subscript. }
\section{Objects from the Class}{
Objects can be created by calls of the form
\code{as("indirectExternalMatrix", vec)} where \code{vec} is an object
of class \code{"externalVector"}.
}
\section{Slots}{
  \describe{
    \item{\code{actual}:}{Object of class \code{"externalVector"}, the
      actual object of which this is a subset }
    \item{\code{index}:}{Object of class \code{"matrixIndex"}, the
      vector subscript to get the subset. }
  }
}
\section{Extends}{
Class \code{"externalVector"}, directly.
}
\section{Methods}{
  \describe{
    \item{e1 op e2}{ For any arithmetic or comparison operation \code{op}
      this returns the result of the operation. Either (or both) of
      \code{e1} and \code{e2} are of class
      \code{"indirectExternalMatrix"}. }
    \item{Complex(z)}{ Return the result of an operation in the
      \code{Complex} group generic. }
    \item{Math(x)}{ Return the result of an operation in the
      \code{Math} group generic. }
    \item{log(x)}{ Return natural logarithm of \code{x}. }
    \item{log(x, base)}{ Return logarithm of \code{x} with base
      \code{base}. }
    \item{x[i, j, \dots, drop=FALSE]}{ Extract a subset of \code{x}
      according to the given subscripts. }
    \item{x[i, j, \dots] <- value}{ Subassign of \code{x} by \code{value}
      according to the given subscripts. This assigns to the correct
      elements of \code{x@actual}. }
    \item{as.Rvector(x)}{ Coerce \code{x} to a basic R vector with same
      names, dimension and dimnames (if any of these are present in
      \code{x}). }
    \item{as.vector(x, mode)}{ Return an R basic vector with mode
      \code{mode}. }
    \item{as.matrix}{ Return a matrix of appropriate mode }.
    \item{length(x)}{ Return the length of \code{x}. }
    \item{length(x) <- value}{ Set the length of \code{x} to \code{value}. }
    \item{dim(x)}{ Return the dimension of \code{x}. }
    \item{dim(x) <- value}{ Set the dimension of \code{x} to \code{value}. }
    \item{dimnames(x)}{ Return the dimnames of \code{x}. }
    \item{dimnames(x) <- value}{ Set the dimnames of \code{x} to \code{value}. }
    \item{getExternalStorage(x)}{ Return the \code{storage} slot of
      associated with \code{x@actual}. }
    \item{is.finite(x)}{ Return a vector of logicals with same
      length as \code{x} indicating if an element of \code{x} is finite. }
    \item{is.infinite(x)}{ Return a vector of logicals with same
      length as \code{x} indicating if an element of \code{x} is infinite. }
    \item{is.na(x)}{ Return a vector of logicals with same
      length as \code{x} indicating if an element of \code{x} is \code{NA}. }
    \item{is.nan(x)}{ Return a vector of logicals with same
      length as \code{x} indicating if an element of \code{x} is \code{NaN}. }
    \item{force}{ return the result of \code{force(x@actual)[x@index]}. }
    \item{all(x, \dots, na.rm)}{ Are all the values \code{TRUE}? }
    \item{any(x, \dots, na.rm)}{ Are any of the values \code{TRUE}? }
    \item{min(x, \dots, na.rm)}{ Minima of the values. }
    \item{max(x, \dots, na.rm)}{ Maxima of the values. }
    \item{range(x, \dots, na.rm)}{ Range of the values. }
    \item{prod(x, \dots, na.rm)}{ Product of the values. }
    \item{sum(x, \dots, na.rm)}{ Sum of the values. }
  }
}
\seealso{
  \code{\link{indirectExternalVector-class}} for the matrix class
  corresponding to this.
}
\keyword{classes}

\eof
\name{indirectExternalVector-class}
\docType{class}
\alias{indirectExternalVector-class}
\alias{Arith,ANY,indirectExternalVector-method}
\alias{Arith,indirectExternalVector,ANY-method}
\alias{Arith,indirectExternalVector,indirectExternalVector-method}
\alias{Compare,ANY,indirectExternalVector-method}
\alias{Compare,indirectExternalVector,ANY-method}
\alias{Compare,indirectExternalVector,indirectExternalVector-method}
\alias{Complex,indirectExternalVector-method}
\alias{Math,indirectExternalVector-method}
\alias{Ops,ANY,indirectExternalVector-method}
\alias{Ops,indirectExternalVector,ANY-method}
\alias{Ops,indirectExternalVector,indirectExternalVector-method}
\alias{Summary,indirectExternalVector,ANY-method}
\alias{[,indirectExternalVector,missing,missing,ANY-method}
\alias{[,indirectExternalVector,ANY,missing,ANY-method}
\alias{[<-,indirectExternalVector,missing,missing-method}
\alias{[<-,indirectExternalVector,ANY,missing-method}
\alias{allNA,indirectExternalVector-method}
\alias{anyNA,indirectExternalVector-method}
\alias{as.Rvector,indirectExternalVector-method}
\alias{as.matrix,indirectExternalVector-method}
\alias{as.vector,indirectExternalVector,missing-method}
\alias{as.vector,indirectExternalVector,ANY-method}
\alias{coerce,externalVector,indirectExternalVector-method}
\alias{coerce,vector,indirectExternalVector-method}
\alias{coerce,indirectExternalVector,vector-method}
\alias{coerce,indirectExternalVector,matrix-method}
\alias{dim,indirectExternalVector-method}
\alias{dim<-,indirectExternalVector,ANY-method}
\alias{dimnames,indirectExternalVector-method}
\alias{dimnames<-,indirectExternalVector-method}
\alias{force,indirectExternalVector-method}
\alias{getExternalStorage,indirectExternalVector-method}
\alias{is.finite,indirectExternalVector-method}
\alias{is.infinite,indirectExternalVector-method}
\alias{is.na,indirectExternalVector-method}
\alias{is.nan,indirectExternalVector-method}
\alias{length,indirectExternalVector-method}
\alias{length<-,indirectExternalVector-method}
\alias{log,ANY,indirectExternalVector-method}
\alias{log,indirectExternalVector,ANY-method}
\alias{log,indirectExternalVector,indirectExternalVector-method}
\alias{log,indirectExternalVector,missing-method}
\alias{names,indirectExternalVector-method}
\alias{names<-,indirectExternalVector,NULL-method}
\alias{setNames,indirectExternalVector-method}
\alias{all,indirectExternalVector-method}
\alias{any,indirectExternalVector-method}
\alias{min,indirectExternalVector-method}
\alias{max,indirectExternalVector-method}
\alias{range,indirectExternalVector-method}
\alias{sum,indirectExternalVector-method}
\alias{prod,indirectExternalVector-method}

\title{Class "indirectExternalVector", class for indirect vector subset of an
  externalVector object. }
\description{ Objects from this class store an object of class
  \code{"externalVector"}  and a vector subscript for that object and
  behaves as if this is an object created by applying that subscript. }
\section{Objects from the Class}{
Objects can be created by calls of the form
\code{as("indirectExternalVector", vec)} where \code{vec} is an object
of class \code{"externalVector"}.
}
\section{Slots}{
  \describe{
    \item{\code{actual}:}{Object of class \code{"externalVector"}, the
      actual object of which this is a subset }
    \item{\code{index}:}{Object of class \code{"vectorIndex"}, the
      vector subscript to get the subset. }
  }
}
\section{Extends}{
Class \code{"externalVector"}, directly.
}
\section{Methods}{
  \describe{
    \item{e1 op e2}{ For any arithmetic or comparison operation \code{op}
      this returns the result of the operation. Either (or both) of
      \code{e1} and \code{e2} are of class
      \code{"indirectExternalVector"}. }
    \item{Complex(z)}{ Return the result of an operation in the
      \code{Complex} group generic. }
    \item{Math(x)}{ Return the result of an operation in the
      \code{Math} group generic. }
    \item{log(x)}{ Return natural logarithm of \code{x}. }
    \item{log(x, base)}{ Return logarithm of \code{x} with base
      \code{base}. }
    \item{x[i, j, \dots, drop=FALSE]}{ Extract a subset of \code{x}
      according to the given subscripts. }
    \item{x[i, j, \dots] <- value}{ Subassign of \code{x} by \code{value}
      according to the given subscripts. This assigns to the correct
      elements of \code{x@actual}. }
    \item{as.Rvector(x)}{ Coerce \code{x} to a basic R vector with same
      names, dimension and dimnames (if any of these are present in
      \code{x}). }
    \item{as.vector(x, mode)}{ Return an R basic vector with mode
      \code{mode}. }
    \item{as.matrix}{ Return a matrix of appropriate mode }.
    \item{length(x)}{ Return the length of \code{x}. }
    \item{length(x) <- value}{ Set the length of \code{x} to \code{value}. }
    \item{dim(x)}{ Return the dimension of \code{x}. }
    \item{dim(x) <- value}{ Set the dimension of \code{x} to \code{value}. }
    \item{dimnames(x)}{ Return the dimnames of \code{x}. }
    \item{dimnames(x) <- value}{ Set the dimnames of \code{x} to \code{value}. }
    \item{getExternalStorage(x)}{ Return the \code{storage} slot of
      associated with \code{x@actual}. }
    \item{is.finite(x)}{ Return a vector of logicals with same
      length as \code{x} indicating if an element of \code{x} is finite. }
    \item{is.infinite(x)}{ Return a vector of logicals with same
      length as \code{x} indicating if an element of \code{x} is infinite. }
    \item{is.na(x)}{ Return a vector of logicals with same
      length as \code{x} indicating if an element of \code{x} is \code{NA}. }
    \item{is.nan(x)}{ Return a vector of logicals with same
      length as \code{x} indicating if an element of \code{x} is \code{NaN}. }
    \item{setNames(x, value)}{ Set the names of \code{x} to \code{value}
      without any error checking or coercion. Return the modified
      \code{x}. }
    \item{force}{ return the result of \code{force(x@actual)[x@index]}. }
    \item{all(x, \dots, na.rm)}{ Are all the values \code{TRUE}? }
    \item{any(x, \dots, na.rm)}{ Are any of the values \code{TRUE}? }
    \item{min(x, \dots, na.rm)}{ Minima of the values. }
    \item{max(x, \dots, na.rm)}{ Maxima of the values. }
    \item{range(x, \dots, na.rm)}{ Range of the values. }
    \item{prod(x, \dots, na.rm)}{ Product of the values. }
    \item{sum(x, \dots, na.rm)}{ Sum of the values. }
  }
}
\seealso{
  \code{\link{indirectExternalMatrix-class}} for the matrix class
  corresponding to this.
}
\keyword{classes}

\eof
\name{internalType}
\alias{internalType}
\title{ Get a vector object similar to the one stored inside an external
  storage object}
\description{
  Get a vector object similar to the one stored inside an external
  storage object.
}
\usage{
internalType(x)
}
\arguments{
  \item{x}{ An "externalStorage" object. }
}
\value{
  An \R basic vector object representing the kind of vector object
  stored in the \code{x}.
}
\seealso{ \code{\link{externalStorage-class}} }
\keyword{classes}

\eof
\name{makeSlice}
\alias{makeSlice}
\title{ Create a new object of class "sliceIndex" }
\description{
  Create a new object of class "sliceIndex"
}
\usage{
makeSlice(start = 1, length.slice = 1, stride = 1)
}
\arguments{
  \item{start}{ Start of the index. }
  \item{length.slice}{ Length of the index. }
  \item{stride}{ Difference between consecutive elements of the index. }
}
\value{
  An object of class "sliceIndex".
}
\seealso{\code{\link{sliceIndex-class}}}
\examples{
x <- makeSlice(0, 4, 2)
as(x, "positiveIndex")
}
\keyword{classes}

\eof
\name{matrixIndex-class}
\docType{class}
\alias{matrixIndex-class}
\alias{[,matrixIndex,ANY,ANY,ANY-method}
\alias{[[,matrixIndex,numeric,missing-method}
\alias{allNA,matrixIndex-method}
\alias{anyNA,matrixIndex-method}
\alias{coerce,matrixIndex,vectorIndex-method}
\alias{dim,matrixIndex-method}
\alias{dimnames,matrixIndex-method}
\alias{length,matrixIndex-method}
\alias{names,matrixIndex-method}
\alias{setDimNames,matrixIndex-method}
\alias{setNames,matrixIndex-method}

\title{Class "matrixIndex", index class representing a set of row and
  column subscripts for a matrix }
\description{ Objects of class \code{"matrixIndex"} represent subscripts
  corresponding to a submatrix of some matrix of a given size. Like the
  \code{"vectorIndex"} subclasses, this is a class for internal use. }
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("matrixIndex",
  row, col, origdim, Names)}.
}
\section{Slots}{
  \describe{
    \item{\code{row}:}{Object of class \code{"vectorIndex"}, the row
      subscript. }
    \item{\code{col}:}{Object of class \code{"vectorIndex"}, the column
      subscript. }
    \item{\code{origdim}:}{Object of class \code{"integer"}, the
      dimension of the full matrix. }
    \item{\code{Names}:}{Object of class \code{"vectorNamesType"}, names
      for the submatrix elements when the submatrix is treated as
      vectors. }
  }
}
\section{Extends}{
Class \code{"externalVector"}, directly.
}
\section{Methods}{
  Signature components for the methods are:

  \tabular{ll}{
    x \tab The class "allIndex"\cr
    scalar \tab Length one positive "integer"\cr
    i \tab The class "ANY"\cr
    j \tab The class "ANY"\cr
    drop \tab The class "logical"\cr
    value \tab The class "ANY"\cr
    .Object \tab The class "allIndex"\cr
  }

  \describe{
    \item{x[i, j, drop=FALSE]}{ Create another object of cloass
      \code{matrixIndex} with same \code{origdims} represents a further
      submatrix of the full matrix. If \code{drop} is \code{TRUE} and at
      least one of the dimensions of the answer is \code{1}, the answer
      is an object of class \code{"vectorIndex"} instead. }
    \item{x[[i]]}{ Get the subscript representing the indices for
      dimension \code{i}. }
    \item{dim(x)}{ Get the dimension of \code{x}. }
    \item{dimnames(x)}{ Get the dimnames of \code{x}. }
    \item{length(x)}{ Get the length of \code{x} considered as a
      vector. This is same as \code{prod(dim(x))}. }
    \item{names(x)}{ Get the names of \code{x}. }
    \item{setDimNames(x, value)}{ Set the dimnames of \code{x} and
      return the modified \code{x}. }
    \item{setNames(x, value)}{ Set the names of \code{x} and return the
      modified \code{x}. }
    \item{as(x, "vectorIndex")}{ Coerce x to an object of class
      "vectorIndex". }
  }
}
\seealso{
  \code{\link{vectorIndex-class}} for the class of row and column subscripts.
}
\keyword{classes}

\eof
\name{nativeStorageMethods-class}
\docType{class}
\alias{nativeStorageMethods-class}
\alias{allocatedSize,nativeStorageMethods-method}
\alias{allocator,nativeStorageMethods-method}
\alias{getPointer,nativeStorageMethods-method}
\alias{initializeResource,nativeStorageMethods-method}

\title{Class "nativeStorageMethods", class holding an external pointer
  to a C storeageMethods structure. }
\description{ This simple class has only one purpose - holding an
  external pointer that contains a pointer to a C
  \code{storageMethods} structure. The memory for the C structure is
  allocated by the "gcAllocator" memory allocator. }
\section{Objects from the Class}{
  Objects can be created by calls of the form
  \code{new("nativeStorageMethods", \dots)}. For details on how the
  \code{\dots} argument to \code{new} works, see documentation for the
  \code{"initialize"} method for class \code{"externalResource"}.
  
  The \code{ptr} slot in a newly created object contains
  pointer to an uninitialized C \code{storageMethods} structure.
}
\section{Slots}{
  \describe{
    \item{\code{ptr}:}{Object of class \code{"externalptr"} }
  }
}
\section{Extends}{
Class \code{"externalResource"}, directly.
}
\section{Methods}{
  Signature components of the methods:

  \tabular{ll}{
    resource \tab The class "nativeStorageMethods"\cr
    ptr \tab The class "externalptr"\cr
    size \tab The class "ANY"\cr
    type \tab The class "ANY"\cr
  }

  Description of the methods:

  \describe{
    \item{allocatedSize(resource):}{ Returns the size of the C
      \code{storageMethods} structure in bytes. }
    \item{allocator(resource):}{ Returns an object of class
      \code{"gcAllocator"}. }
    \item{getPointer(reource):}{ Return the \code{ptr} slot. }
    \item{initializeResource(resource, ptr, size, type, \dots):}{
      Sets the \code{ptr} }
  }
}
\seealso{
  \code{\link{nativeStorageMethodsList-class}}
  for how this class is used.
  
  \code{\link{externalResource-class}} for more on the super class of
  this class.
}
\keyword{classes}
\keyword{methods}

\eof
\name{nativeStorageMethodsList-class}
\docType{class}
\alias{nativeStorageMethodsList-class}
\alias{initialize,nativeStorageMethodsList-method}

\title{Class "nativeStorageMethodsList", contains objects from class
  "nativeStorageMethods"}
\description{ The class \code{"nativeStorageMethodsList"} contains a
  number of slots, all from class
  \code{"nativeStorageMethods"}. Usually, each object of class
  \code{"nativeStorageMethodsList"} is associated with a particular
  subclass of \code{"externalSTorage"}.
  
  Objects from this class are never visible to the user.}
\section{Objects from the Class}{
  Objects can be created by calls of the form
  \code{new("nativeStorageMethodsList")}. Any extra arguments are ignored.
}
\section{Slots}{
  All slots in this class are objects of class \code{"nativeStorageMethods"}.
  \describe{
    \item{\code{allMethods}:}{ Contains the C \code{storageMethods}
      structure with all methods registered (directly or indirectly) for
      the associated \code{"externalStorage"} class. }
    \item{\code{logicalMethods}:}{ Object returned by
      \code{getExternalStorageMethods} for the associated
      \code{"externalStorage"} class and type \code{"logical"}. }
    \item{\code{integerMethods}:}{ Object returned by
      \code{getExternalStorageMethods} for the associated
      \code{"externalStorage"} class and type \code{"integer"}. }
    \item{\code{numericMethods}:}{ Object returned by
      \code{getExternalStorageMethods} for the associated
      \code{"externalStorage"} class and type \code{"numeric"}. }
    \item{\code{complexMethods}:}{ Object returned by
      \code{getExternalStorageMethods} for the associated
      \code{"externalStorage"} class and type \code{"complex"}. }
    \item{\code{characterMethods}:}{ Object returned by
      \code{getExternalStorageMethods} for the associated
      \code{"externalStorage"} class and type \code{"character"}. }
    \item{\code{listMethods}:}{ Object returned by
      \code{getExternalStorageMethods} for the associated
      \code{"externalStorage"} class and type \code{"list"}. }
  }
}
\section{Methods}{
  \describe{
    \item{initialize(.Object, \dots)}{ Initializer method for class
      \code{"nativeStorageMethodsList"}. This initializes each of the
      slots of the class. Any extra argument in \code{\dots} is
      ignored. }
  }
}
\keyword{classes}

\eof
\name{nonSlotAttributes}
\alias{nonSlotAttributes}
\alias{nonSlotAttributes<-}
\title{ Functions to extract and set attributes which are not slots. }
\description{
  These functions access an object's attribute list. The first form
  returns the an object's attribute list after removing any attribute
  which is also a slot for the object. The assignment form
  makes the list on the right-hand side of the assignment the
  object's attribute list (if appropriate).
}
\usage{
nonSlotAttributes(x)
nonSlotAttributes(x) <- value
}
\arguments{
  \item{x}{ An object. }
  \item{value}{ An appropriate attribute list, or \code{NULL}. }
}
\seealso{ \code{\link{attributes}} and \code{\link{attributes<-}} }
\keyword{attribute}

\eof
\name{noneIndex-class}
\docType{class}
\alias{noneIndex-class}
\alias{Arith,numeric,noneIndex-method}
\alias{Arith,noneIndex,numeric-method}
\alias{[,noneIndex,missing,missing,missing-method}
\alias{[,noneIndex,ANY,missing,missing-method}
\alias{[,noneIndex,logical,missing,missing-method}
\alias{[<-,noneIndex,ANY,ANY,ANY-method}
\alias{[[<-,noneIndex-method}
\alias{allNA,noneIndex-method}
\alias{anyNA,noneIndex-method}
\alias{initialize,noneIndex-method}
\alias{length,noneIndex-method}
\alias{length<-,noneIndex-method}
\alias{dim<-,noneIndex,ANY-method}

\title{Class "noneIndex", index class representing a subscript for none
  of the elements of a vector or an array dimension }
\description{ When used as a subscript for a vector object
  or for any dimension of an array object
  this is equivalent to \code{integer(0)}. }
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("noneIndex")}.
Objects from this class can not be modified and they are all equivalent.
}
\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"integer"}, this is
      always equal to \code{integer(0)}. }
  }
}
\section{Extends}{
Class \code{"integer"}, directly.
Class \code{"vectorIndex"}, directly.
}
\section{Methods}{
  Signature components for the methods are:

  \tabular{ll}{
    x \tab The class "allIndex"\cr
    scalar \tab Length one positive "integer"\cr
    i \tab The class "ANY"\cr
    value \tab The class "ANY"\cr
    .Object \tab The class "allIndex"\cr
  }

  \describe{
    \item{\code{x+scalar}:}{ Add \code{scalar} to \code{x}.
      Result is \code{x}. }
    \item{\code{scalar+x}:}{ Add \code{scalar} to \code{x}.
      Result is \code{x}. }
    \item{\code{x*scalar}:}{ Multiply \code{x} by \code{scalar}.
      Result is \code{x}. }
    \item{\code{scalar*x}:}{ Multiply \code{x} by \code{scalar}.
      Result is \code{x}. }
    \item{dim(x) <- value}{ Set a dimension on \code{x}. If value is of
      length 2, this may create a "matrixIndex". }
    \item{\code{x[i]}:}{ Subset x. The result is an object of class
      \code{"NAIndex"} of length same as
      \code{length(integer(0)[i])}. }
    \item{\code{x[]}:}{ Subset x with the subscript missing. Result is x. }
    \item{\code{x[i] <- value}:}{ Subassign x. Results in an error (as
      objects from class \code{"noneIndex"} are not mutable. }
    \item{\code{x[[i]] <- value}:}{ Subassign x. Results in an error (as
      objects from class \code{"noneIndex"} are not mutable. }
    \item{\code{allNA(x)}:}{ Are all elements of \code{x} \code{NA}? Result is
      always \code{FALSE}. }
    \item{\code{anyNA(x)}:}{ Is any element of \code{x} \code{NA}? Result is
      always \code{FALSE}. }
    \item{\code{initialize(.Object)}:}{Initialize an object from class
      \code{"allIndex"}. }
    \item{\code{length(x)}:}{ Return the length of \code{x} which is always
      \code{0}. }
    \item{\code{length(x) <- value}:}{ Modify the length of \code{x}. Always
      results in an error. }
  }
}
\seealso{
  \code{\link{vectorIndex-class}} for the super class of \code{"noneIndex"}.
}
\keyword{classes}

\eof
\name{positiveIndex-class}
\docType{class}
\alias{positiveIndex-class}
\alias{Arith,positiveIndex,numeric-method}
\alias{Arith,numeric,positiveIndex-method}
\alias{[,positiveIndex,sliceIndex,missing,ANY-method}
\alias{[,positiveIndex,positiveIndex,missing,ANY-method}
\alias{[,positiveIndex,ANY,missing,ANY-method}
\alias{allNA,positiveIndex-method}
\alias{anyNA,positiveIndex-method}
\alias{as.integer,positiveIndex-method}
\alias{coerce,integer,positiveIndex-method}
\alias{coerce,numeric,positiveIndex-method}
\alias{initialize,positiveIndex-method}
\alias{dim<-,positiveIndex,ANY-method}

\title{Class "positiveIndex", index class representing a sequence of
  non-negative integers }
\description{ This represents a subscript of arbitrary non-negative
  integers. }
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("positiveIndex",
    val)}. If \code{val} is missing, the result is an object of class
  \code{"positiveIndex"} of length \code{0}. Otherwise, it is coerced
  to an integer and an object of class  \code{"positiveIndex"} is
  created which has the same \code{.Data} slot as \code{val}. The
  \code{rangeIndex} and \code{noNA} slots are calculated by the
  initialization code. }
\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"integer"}, the actual
      index values. }
    \item{\code{rangeIndex}:}{Object of class \code{"integer"} of length
      \code{2}, same as \code{range(.Data)}. }
    \item{\code{noNA}:}{Object of class \code{"logical"} of length 1,
      same as \code{!any(is.na(.Data))}. }
  }
}
\section{Extends}{
Class \code{"integer"}, directly.
Class \code{"vectorIndex"}, directly.
}
\section{Methods}{
  Signature components for the methods are:

  \tabular{ll}{
    x \tab The class "allIndex"\cr
    scalar \tab Length one positive "integer"\cr
    i \tab The class "ANY"\cr
    value \tab The class "ANY"\cr
    .Object \tab The class "allIndex"\cr
  }

  \describe{
    \item{\code{x+scalar}}{ Add \code{scalar} to \code{x}.
      Result is \code{x} with \code{x@.Data} replaced by
      \code{x@.Data+scalar}. }
    \item{\code{scalar+x}}{ Add \code{scalar} to \code{x}.
      Result is \code{x} with \code{x@.Data} replaced by
      \code{x@.Data+scalar}. }
    \item{\code{x*scalar}}{ Multiply \code{x} by \code{scalar}.
      Result is \code{x} with \code{x@.Data} replaced by
      \code{x@.Data*scalar}. }
    \item{\code{scalar*x}}{ Multiply \code{x} by \code{scalar}.
      Result is \code{x} with \code{x@.Data} replaced by
      \code{x@.Data*scalar}. }
    \item{dim(x) <- value}{ Set a dimension on \code{x}. If value is of
      length 2, this may create a "matrixIndex". }
    \item{\code{x[i]}}{ Subset x. If length of \code{i} is \code{0}, the result
      is an object of class \code{"noneIndex"}. Otherwise, the result is
      an object of class \code{"sliceIndex"} or of class
      \code{"positiveIndex"}. }
    \item{\code{x[]}}{ Subset x with the subscript missing. Result is x. }
    \item{\code{allNA(x)}}{ Are all elements of \code{x} \code{NA}? Result is
      \code{all(is.na(x@.Data))}. }
    \item{\code{anyNA(x)}}{ Is any element of \code{x} \code{NA}? Result is
      \code{any(is.na(x@.Data))}. }
    \item{\code{as.integer(x)}}{ Convert \code{x} to an R subscript -
      adds 1 to each element of \code{x@.Data}. }
    \item{\code{initialize(.Object, .Data)}}{ Initialize \code{.Object}. }
  }
}
\keyword{classes}

\eof
\name{rebind}
\alias{rebind}
\title{ Allocate a new external vector with external storage }
\description{
  This function allocates a new "externalVectorWithStorage" object with
  possibly new length, type, names, dimension or dimnames.
}
\usage{
rebind(x, length.x = length(x), type = x@storage@type,
       names.x = x@Names, dim.x = x@Dim,
       dimnames.x = x@DimNames, \dots, arrayOnly = FALSE)
}
\arguments{
  \item{x}{ An object of class "externalVectorWithStorage". }
  \item{length.x}{ New length. }
  \item{type}{ New type. }
  \item{names.x}{ New names. }
  \item{dim.x}{ New dimension. }
  \item{dimnames.x}{ New dimnames. }
  \item{\dots}{Other arguments passed to initializer of the class of
    \code{x}.}
  \item{arrayOnly}{ Just create the data part with no names, dimensions
    or dimnames. }
}
\value{
  A new object of same class as \code{x}.
}
\seealso{ \code{\link{externalVectorWithStorage-class}} }
\keyword{classes}

\eof
\name{setDimNames}
\alias{setDimNames}
\alias{setNames}
\title{ Set names and dimnames }
\description{
  These generics set the names and the dimnames of an object without
  any sanity checks.
}
\usage{
setNames(object, nm)
setDimNames(object, dnm)
}
\arguments{
  \item{object}{ An object. }
  \item{nm}{ The new names. }
  \item{dnm}{ The new dimnames. }
}
\value{
  The \code{object} after modification.
}
\seealso{ \code{\link{names<-}}, \code{\link{dimnames<-}} }
\keyword{attribute}

\eof
\name{setExternalStorageClass}
\alias{setExternalStorageClass}
\title{ Create subclass of externalStorage }
\description{
  Function to create a subclass of "externalStorage" and associate
  native C methods with the class.
}
\usage{
setExternalStorageClass(Class, methodNames, \dots, contains = "externalStorage", where = topenv(parent.frame()))
}
\arguments{
  \item{Class}{ Character string, name for the class. }
  \item{methodNames}{ Character vector, names of the native C
    functions for use as methods (see Adding Native Methods below). }
  \item{\dots}{ Other arguments passed to \code{setClass}. }
  \item{contains}{ what classes does this class extend?  (These are called
    superclasses in some languages.) Should have at least one class
    name which is a subclass of "externalStorage". }
  \item{where}{ The environment in which to
    store or remove the definition.  Defaults to the top-level
    environment of the calling function (the global environment
    for ordinary computations, but the environment or namespace
    of a package when loading that package). }
}
\details{
  This function does two things. It creates a subclass of
  "externalStorage". It also associates a set of given C methods as
  native methods for this subclass of "externalStorage".
}
\section{Adding Native Methods}{
  To make the external vectors as efficient as possible, the interface
  between the storage method classes and the
  "externalVectorWithSTorage" class uses a set of C function pointers
  for each subclass of "externalStorage". There are two types of
  methods - those which are for a particular type of vector and those
  which are not for a particular type. For a type \code{xxx} where
  \code{xxx} is one of \code{logical}, \code{integer}, \code{numeric},
  \code{complex} or \code{character}, the type specific functions are
  \tabular{ll}{
    xxxGetElt \tab get an element from vector of type xxx\cr
    xxxSetElt \tab set an element in a vector of type xxx\cr
    xxxSubset \tab subset a vector of type xxx\cr
    xxxSubassign \tab subassign a vector of type xxx\cr
    xxxGetMatrixElt \tab get an element from matrix of type xxx\cr
    xxxSetMatrixElt \tab set an element in a matrix of type xxx\cr
    xxxMatrixSubset \tab subset a matrix of type xxx\cr
    xxxMatrixSubassign \tab subassign a matrix of type xxx\cr
  }
  The functions not specific to any type are
  \tabular{ll}{
    alloc \tab allocate a new storage object\cr
    size \tab return the length of the vector in the storage object\cr
    resize \tab modify the length of the vector in the storage object\cr
  }
  The subset and subassign functions for vectors and matrices have
  reasonable default implementations in terms of the element get/set functions
  - so they need not be implemented for each "externalStorage" subclass.
  
  The \code{methodNames} argument to \code{setExternalStorageClass}
  must be used to register any C function for use as a particular
  method. This argument must be a character vector with elements of the
  form \code{methodName=functionName} where \code{methodName} is one of
  the method names given in the tables above and function name is a C
  function name which has been registered as a C function with R using
  the foreign function registration mechanism. See the chapter on
  \dQuote{System and foreign language interfaces} in
  \dQuote{Writing \R Extensions} in the \file{doc/manual}
  subdirectory of the \R source tree for more details on registering C
  functions.
}
\seealso{ \code{\link[methods]{setClass}} for possible arguments in \dots.
  \code{\link{externalStorage-class}} for the "externalStorage" class.
}
\examples{
\dontrun{
setExternalStorageClass("simpleStorage", c(logicalGetElt="simpleLogicalGetElt",
                                           logicalSetElt="simpleLogicalSetElt",
                                           numericGetElt="simpleNumericGetElt",
                                           numericSetElt="simpleNumericSetElt",
                                           size="simpleSize",
                                           resize="simpleResize"
                                           alloc="simpleAlloc"),
                        contains = "externalStorage")
}
}
\keyword{classes}
\keyword{programming}

\eof
\name{setVirtualMethod}
\alias{setVirtualMethod}
\title{ Set a virtual method for a given signature }
\description{
  This sets a method for a signature. If there is no generic with the
  given name, it also creates the appropriate generic. If called, the method
  generates an error indicating the classes of the arguments which are
  part of the signature.
}
\usage{
setVirtualMethod(name, signature.virtual, signature.nonvirtual = character(), \dots, where = topenv(parent.frame()))
}
\arguments{
  \item{name}{ The character-string name of the generic function. }
  \item{signature.virtual}{ Signature of the arguments for which the
    virtual method is being set. All of these arguments must have a
    virtual class in the signature. }
  \item{signature.nonvirtual}{ Signature for other arguments. Any
    ommitted argument is assumed to have class "ANY". }
  \item{\dots}{ Other arguments. See details. }
  \item{where}{ The database in which to store the definition of the
    method. }
}
\details{
  If the generic for \code{name} is not defined, an attempt is made to
  create it with \code{setGeneric(name, \dots, where = where)}.
}
\value{
  This method exists for its side-effect of setting up the virtual
  method.
  
  The return value is the result from the call to \code{setMethod} to
  create the virtual method.
}
\seealso{ \code{\link[methods]{setGeneric}}, \code{\link[methods]{setMethod}} }
\examples{
setClass("myMatrix")
setVirtualMethod("dim", "myMatrix")
setVirtualMethod("dimnames", "myMatrix")
setClass("myMatrix2", representation(val="numeric",
                                     d = "integer",
                                     dn = "list"),
         contains="myMatrix")
x <- new("myMatrix2", val=1:4, d = as.integer(c(2, 2)),
         dn = list(LETTERS[1:2], letters[1:2]))
## A call dim(x) or dimnames(x) would generate an error here
setMethod("dim", "myMatrix", function(x) x@d)
setMethod("dimnames", "myMatrix", function(x) x@dn)
dim(x)
dimnames(x)
}
\keyword{methods}

\eof
\name{simpleStorage-class}
\docType{class}
\alias{simpleStorage-class}
\alias{allocatedSize,simpleStorage-method}
\alias{allocatedType,simpleStorage-method}

\title{Class "simpleStorage", an external storage class that uses
  reference to R basic vectors as storage }
\description{ The class \code{"simpleStorage"} is an implementation of
  the \code{"externalStorage"} class. The actual storage for the vector
  object is contained in an R basic vector referenced through the
  \code{ptr} slot. }
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("simpleStorage",
    type, length)}. See the \code{initialize} method for
  \code{"externalStorage"} for details.
}
\section{Slots}{
  \describe{
    \item{\code{ptr}:}{Object of class \code{"externalptr"}, keeps the
      R basic vector in the protected field. For \code{"logical"},
      \code{"integer"}, \code{"numeric"}, \code{"complex"}, the address
      field of the external pointer also holds a pointer to the data in
      the basic R vector. }
    \item{\code{type}:}{Object of class \code{"vector"}, a vector object
      of length one representing the type of object stored in the
      external storage. Usually one of the basic R vector types. }
    \item{\code{length}:}{Object of class \code{"integer"}, Cached value
      for the length of the vector stored in the external storage. }
  }
}
\section{Extends}{
Class \code{"externalStorage"}, directly.
Class \code{"externalResource"}, by class \code{"externalStorage"}.
}
\section{Methods}{
  Signature components for the methods:

  \tabular{ll}{
    resource \tab The class "simpleStorage"\cr
  }

  Description of the methods:
  
  \describe{
    \item{allocatedSize(resource)}{ Length of the vector to be stored. }
    \item{allocatedType}{ The type of vector to be stored. }
  }
}
\seealso{
  \code{\link{externalStorage-class}} to see details of the super class.
  
  \code{\link{externalVector}} to create \code{"externalVector"}
  objects that use this resource.
}
\keyword{classes}

\eof
\name{sliceIndex-class}
\docType{class}
\alias{sliceIndex-class}
\alias{Arith,sliceIndex,numeric-method}
\alias{Arith,numeric,sliceIndex-method}
\alias{[,sliceIndex,allIndex,missing,ANY-method}
\alias{[,sliceIndex,noneIndex,missing,ANY-method}
\alias{[,sliceIndex,sliceIndex,missing,ANY-method}
\alias{[,sliceIndex,positiveIndex,missing,ANY-method}
\alias{[,sliceIndex,ANY,missing,ANY-method}
\alias{allNA,sliceIndex-method}
\alias{anyNA,sliceIndex-method}
\alias{coerce,sliceIndex,positiveIndex-method}
\alias{length,sliceIndex-method}
\alias{length<-,sliceIndex-method}
\alias{names,sliceIndex-method}
\alias{names<-,sliceIndex,ANY-method}
\alias{names<-,sliceIndex,externalVector-method}
\alias{dim<-,sliceIndex,ANY-method}

\title{Class "sliceIndex", index class representing a sequence of
  non-negative integers }
\description{ This represents a sequence that can be created with
  \code{seq(start, by=stride, length=n)} where \code{star >= 0} and
  \code{start+(n-1)*stride >= 0}. }
\section{Objects from the Class}{
Objects can be created by calls of the form \code{makeSlice(start,
  length, stride)}.
}
\section{Slots}{
  \describe{
    \item{\code{content}:}{Object of class \code{"numeric"} of length
      \code{3}. The first element of \code{content} is the start of the
      index, the second element is the length of the index, and the
      third element is the difference between successive elements of the
      index. }
    \item{\code{Names}:}{Object of class \code{"vectorNamesType"}, this
      contains names associated with the elements of the index (if any). }
  }
}
\section{Extends}{
Class \code{"vectorIndex"}, directly.
}
\section{Methods}{
  Signature components for the methods are:

  \tabular{ll}{
    x \tab The class "allIndex"\cr
    scalar \tab Length one positive "integer"\cr
    i \tab The class "ANY"\cr
    value \tab The class "ANY"\cr
    .Object \tab The class "allIndex"\cr
  }

  \describe{
    \item{\code{x+scalar}:}{ Add \code{scalar} to \code{x}.
      Result is \code{x} with \code{x@content[1]} replaced by
      \code{x@content[1]+scalar}. }
    \item{\code{scalar+x}:}{ Add \code{scalar} to \code{x}.
      Result is \code{x} with \code{x@content[1]} replaced by
      \code{x@content[1]+scalar}. }
    \item{\code{x*scalar}:}{ Multiply \code{x} by \code{scalar}.
      Result is \code{x} with \code{x@content[3]} replaced by
      \code{x@content[3]*scalar}. }
    \item{\code{scalar*x}:}{ Multiply \code{x} by \code{scalar}.
      Result is \code{x} with \code{x@content[3]} replaced by
      \code{x@content[3]*scalar}. }
    \item{\code{x[i]}:}{ Subset x. If length of \code{i} is \code{0}, the result
      is an object of class \code{"noneIndex"}. Otherwise, the result is
      an object of class \code{"sliceIndex"} or of class
      \code{"positiveIndex"}. }
    \item{dim(x) <- value}{ Set a dimension on \code{x}. If value is of
      length 2, this may create a "matrixIndex". }
    \item{\code{x[]}:}{ Subset x with the subscript missing. Result is x. }
    \item{\code{allNA(x)}:}{ Are all elements of \code{x} \code{NA}? Result is
      always \code{FALSE}. }
    \item{\code{anyNA(x)}:}{ Is any element of \code{x} \code{NA}? Result is
      always \code{FALSE}. }
    \item{\code{length(x)}:}{ Return the length of \code{x}. }
    \item{\code{length(x) <- value}:}{ Modify the length of \code{x}. If
      \code{value} is \code{0}, makes \code{x} an object of class
      \code{"noneIndex"}. Otherwise, modifies length of \code{x} to
      \code{value}. }
    \item{\code{names(x)}:}{ Returns the \code{Names} slot of \code{x}. }
    \item{\code{names(x) <- value }:}{ Sets the \code{Names} slot of \code{x} to
      \code{value}. }
  }
}
\keyword{classes}

\eof
\name{union-class}
\docType{class}
\alias{listOrNull-class}
\alias{integerOrNull-class}
\alias{vectorNamesType-class}
\title{Some useful class unions}
\description{ The class "listOrNull" is a union of the classes "list"
  and "NULL". The class "listOrNull" is a union of the classes "list"
  and "NULL". The class "vectorNamesType" is a union of the classes
  "character", "NULL" and "externalVectorWithStorage".
}
\section{Objects from the Class}{These are virtual classes: no objects may be created from them.}
\section{Methods}{
No methods defined with class "listOrNull", "integerOrNull" or
"vectorNamesType" in the signature.
}
\keyword{classes}

\eof
\name{var}
\title{Correlation, Variance and Covariance (Matrices)}
\usage{
var(x, y, na.rm = FALSE, use)

cov(x, y, use = "all.obs",
    method = c("pearson", "kendall", "spearman"))

cor(x, y, use = "all.obs",
     method = c("pearson", "kendall", "spearman"))

cov2cor(V)
}
\alias{var}
\alias{cov}
\alias{cor}
\alias{cov2cor}
\alias{cor,ANY,ANY-method}
\alias{cov,ANY,ANY-method}
\alias{var,ANY,ANY-method}
\alias{cor,ANY,missing-method}
\alias{cov,ANY,missing-method}
\alias{var,ANY,missing-method}
\description{
  \code{var}, \code{cov} and \code{cor} compute the variance of \code{x}
  and the covariance or correlation of \code{x} and \code{y} if these
  are vectors.	If \code{x} and \code{y} are matrices then the
  covariances (or correlations) between the columns of \code{x} and the
  columns of \code{y} are computed.

  \code{cov2cor} scales a covariance matrix into the corresponding
  correlation matrix \emph{efficiently}.
}
\arguments{
  \item{x}{a numeric vector, matrix or data frame.}
  \item{y}{\code{NULL} (default) or a vector, matrix or data frame with
    compatible dimensions to \code{x}.	The default is equivalent to
    \code{y = x} (but more efficient).}
  \item{na.rm}{logical. Should missing values be removed?}
  \item{use}{an optional character string giving a
    method for computing covariances in the presence
    of missing values.	This must be (an abbreviation of) one of the strings
    \code{"all.obs"}, \code{"complete.obs"} or \code{"pairwise.complete.obs"}.}
  \item{method}{a character string indicating which correlation
    coefficient (or covariance) is to be computed.  One of
    \code{"pearson"} (default), \code{"kendall"}, or \code{"spearman"},
    can be abbreviated.}
  \item{V}{symmetric numeric matrix, usually positive definite such as a
    covariance matrix.}
}
\value{For \code{r <- cor(*, use = "all.obs")}, it is now guaranteed that
  \code{all(r <= 1)}.
}
\details{
  For \code{cov} and \code{cor} one must \emph{either} give a matrix or
  data frame for \code{x} \emph{or} give both \code{x} and \code{y}.

  \code{var} is just another interface to \code{cov}, where
  \code{na.rm} is used to determine the default for \code{use} when that
  is unspecified.  If \code{na.rm} is \code{TRUE} then the complete
  observations (rows) are used (\code{use = "complete"}) to compute the
  variance.  Otherwise (\code{use = "all"}), \code{var} will give an
  error if there are missing values.

  If \code{use} is \code{"all.obs"}, then the presence
  of missing observations will produce an error.
  If \code{use} is \code{"complete.obs"} then missing values
  are handled by casewise deletion.  Finally, if \code{use} has the
  value \code{"pairwise.complete.obs"} then the correlation between
  each pair of variables is computed using all complete pairs
  of observations on those variables.
  This can result in covariance or correlation matrices which are not
  positive semidefinite.

  The denominator \eqn{n - 1} is used which gives an unbiased estimator
  of the (co)variance for i.i.d. observations.
  These functions return \code{\link{NA}} when there is only one
  observation (whereas S-PLUS has been returning \code{NaN}), and %from \R 1.2.3
  fail if \code{x} has length zero.

  For \code{cor()}, if \code{method} is \code{"kendall"} or
  \code{"spearman"}, Kendall's \eqn{\tau}{tau} or Spearman's
  \eqn{\rho}{rho} statistic is used to estimate a rank-based measure of
  association.  These are more robust and have be recommended if the
  data do not necessarily come from a bivariate normal distribution.\cr
  For \code{cov()}, a non-Pearson method is unusual but available for
  the sake of completeness.  Note that \code{"spearman"} basically
  computes \code{cor(R(x), R(y))} (or \code{cov(.,.)}) where
  \code{R(u) := rank(u, na.last="keep")}.

  Scaling a covariance matrix into a correlation one can be achieved in
  many ways, mathematically most appealing by multiplication with a
  diagonal matrix from left and right, or more efficiently by using
  \code{\link{sweep}(.., FUN = "/")} twice.  The \code{cov2cor} function
  is even a bit more efficient, and provided mostly for didactical
  reasons.
}
\references{
  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth \& Brooks/Cole.
}
\seealso{
  \code{\link{cor.test}} (package \pkg{ctest}) for confidence
  intervals (and tests).
  
  \code{\link{cov.wt}} for \emph{weighted} covariance computation.

  \code{\link{sd}} for standard deviation (vectors).
}
\examples{
x <- externalNumeric(10)
x[] <- 1:10
var(x)# 9.166667

var(x[1:5],x[1:5])# 2.5

## Two simple vectors
cor(x,x+1)# == 1

}
\keyword{univar}
\keyword{multivariate}
\keyword{array}

\eof
\name{vectorIndex-class}
\docType{class}
\alias{vectorIndex-class}
\alias{Arith,ANY,vectorIndex-method}
\alias{Arith,vectorIndex,ANY-method}

\title{Class "vectorIndex", base class for index classes used to
  represent vector subscripts }
\description{ Together, all subclasses of this class represent the
  possible representations of vector subscripts and subscripts for
  individual array dimensions.
  
  Usually users would not create objects from any of the subclasses of
  \code{"vectorIndex"} directly. }
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}
\section{Methods}{
No methods defined with class "vectorIndex" in the signature.
}
\seealso{
  \code{\link{allIndex-class}}, \code{\link{noneIndex-class}},
  \code{\link{NAIndex-class}}, \code{\link{sliceIndex-class}},
  \code{\link{positiveIndex-class}}, all of which are subclasses
  of\code{"vectorIndex"}.
}
\keyword{classes}

\eof
